# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .people_payment import PeoplePayment
from .people_client_legal_entity import PeopleClientLegalEntity
from .identifier_value import IdentifierValue, IdentifierValueGuard


@JsonMap({"id_": "id"})
class EmploymentDetailTeam(BaseModel):
    """The team to which the employee belongs.

    :param id_: id_, defaults to None
    :type id_: IdentifierValue, optional
    :param name: The team name., defaults to None
    :type name: str, optional
    """

    def __init__(self, id_: IdentifierValue = None, name: str = None):
        if id_ is not None:
            self.id_ = IdentifierValueGuard.return_one_of(id_)
        if name is not None:
            self.name = name


@JsonMap({"id_": "id"})
class EmploymentDetail(BaseModel):
    """EmploymentDetail

    :param id_: Unique identifier for the employment., defaults to None
    :type id_: str, optional
    :param name: Name of the employee., defaults to None
    :type name: str, optional
    :param team: The team to which the employee belongs., defaults to None
    :type team: EmploymentDetailTeam, optional
    :param email: The work email of the employee., defaults to None
    :type email: str, optional
    :param state: The state of employment., defaults to None
    :type state: str, optional
    :param country: The country of employment., defaults to None
    :type country: str, optional
    :param payment: payment, defaults to None
    :type payment: PeoplePayment, optional
    :param is_ended: Whether the employment has ended., defaults to None
    :type is_ended: bool, optional
    :param timezone: The timezone of the employment., defaults to None
    :type timezone: str, optional
    :param job_title: The job title of the employment., defaults to None
    :type job_title: str, optional
    :param seniority: The seniority level of the employment., defaults to None
    :type seniority: str, optional
    :param start_date: The start date of the employment., defaults to None
    :type start_date: str, optional
    :param work_email: The work email of the employment., defaults to None
    :type work_email: str, optional
    :param hiring_type: The type of hiring of the employment., defaults to None
    :type hiring_type: str, optional
    :param hiring_status: The current hiring status of the employment., defaults to None
    :type hiring_status: str, optional
    :param completion_date: The completion date of the employment contract., defaults to None
    :type completion_date: str, optional
    :param contract_status: The status of the employment contract., defaults to None
    :type contract_status: str, optional
    :param voluntarily_left: Whether the employee voluntarily left the employment., defaults to None
    :type voluntarily_left: str, optional
    :param contract_coverage: The coverage of the employment contract., defaults to None
    :type contract_coverage: List[str], optional
    :param new_hiring_status: The new hiring status of the employment., defaults to None
    :type new_hiring_status: str, optional
    :param client_legal_entity: client_legal_entity, defaults to None
    :type client_legal_entity: PeopleClientLegalEntity, optional
    :param has_eor_termination: Whether the employment has an EOR termination., defaults to None
    :type has_eor_termination: str, optional
    :param contract_is_archived: Whether the employment contract is archived., defaults to None
    :type contract_is_archived: bool, optional
    :param contract_has_contractor: Whether the contract has a contractor., defaults to None
    :type contract_has_contractor: bool, optional
    :param is_user_contract_deleted: Whether the user contract is deleted., defaults to None
    :type is_user_contract_deleted: bool, optional
    :param hris_direct_employee_invitation: The HRIS direct employee invitation status., defaults to None
    :type hris_direct_employee_invitation: str, optional
    """

    def __init__(
        self,
        id_: str = None,
        name: str = None,
        team: EmploymentDetailTeam = None,
        email: str = None,
        state: str = None,
        country: str = None,
        payment: PeoplePayment = None,
        is_ended: bool = None,
        timezone: str = None,
        job_title: str = None,
        seniority: str = None,
        start_date: str = None,
        work_email: str = None,
        hiring_type: str = None,
        hiring_status: str = None,
        completion_date: str = None,
        contract_status: str = None,
        voluntarily_left: str = None,
        contract_coverage: List[str] = None,
        new_hiring_status: str = None,
        client_legal_entity: PeopleClientLegalEntity = None,
        has_eor_termination: str = None,
        contract_is_archived: bool = None,
        contract_has_contractor: bool = None,
        is_user_contract_deleted: bool = None,
        hris_direct_employee_invitation: str = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if name is not None:
            self.name = name
        if team is not None:
            self.team = self._define_object(team, EmploymentDetailTeam)
        if email is not None:
            self.email = email
        if state is not None:
            self.state = state
        if country is not None:
            self.country = country
        if payment is not None:
            self.payment = self._define_object(payment, PeoplePayment)
        if is_ended is not None:
            self.is_ended = is_ended
        if timezone is not None:
            self.timezone = timezone
        if job_title is not None:
            self.job_title = job_title
        if seniority is not None:
            self.seniority = seniority
        if start_date is not None:
            self.start_date = start_date
        if work_email is not None:
            self.work_email = work_email
        if hiring_type is not None:
            self.hiring_type = hiring_type
        if hiring_status is not None:
            self.hiring_status = hiring_status
        if completion_date is not None:
            self.completion_date = completion_date
        if contract_status is not None:
            self.contract_status = contract_status
        if voluntarily_left is not None:
            self.voluntarily_left = voluntarily_left
        if contract_coverage is not None:
            self.contract_coverage = contract_coverage
        if new_hiring_status is not None:
            self.new_hiring_status = new_hiring_status
        if client_legal_entity is not None:
            self.client_legal_entity = self._define_object(
                client_legal_entity, PeopleClientLegalEntity
            )
        if has_eor_termination is not None:
            self.has_eor_termination = has_eor_termination
        if contract_is_archived is not None:
            self.contract_is_archived = contract_is_archived
        if contract_has_contractor is not None:
            self.contract_has_contractor = contract_has_contractor
        if is_user_contract_deleted is not None:
            self.is_user_contract_deleted = is_user_contract_deleted
        if hris_direct_employee_invitation is not None:
            self.hris_direct_employee_invitation = hris_direct_employee_invitation



# This file was generated by liblab | https://liblab.com/

import requests

from requests.exceptions import Timeout
from typing import Optional, Tuple
from .base_handler import BaseHandler
from ...transport.request import Request
from ...transport.response import Response
from ...transport.request_error import RequestError


class HttpHandler(BaseHandler):
    """
    Handler for making HTTP requests.
    This handler sends the request to the specified URL and returns the response.

    :ivar int _timeout_in_seconds: The timeout for the HTTP request in seconds.
    """

    def __init__(self):
        """
        Initialize a new instance of HttpHandler.
        """
        super().__init__()
        self._timeout_in_seconds = 60

    def handle(
        self, request: Request
    ) -> Tuple[Optional[Response], Optional[RequestError]]:
        """
        Send the request to the specified URL and return the response.

        :param Request request: The request to send.
        :return: The response and any error that occurred.
        :rtype: Tuple[Optional[Response], Optional[RequestError]]
        """
        try:
            request_args = self._get_request_data(request)

            result = requests.request(
                request.method,
                request.url,
                headers=request.headers,
                timeout=self._timeout_in_seconds,
                **request_args,
            )
            response = Response(result)

            if result.status_code >= 400:
                return None, RequestError(
                    message=f"{result.status_code} error in request to: {request.url}",
                    status_code=result.status_code,
                    response=response,
                )

            return response, None
        except Timeout:
            return None, RequestError("Request timed out")

    def _get_request_data(self, request: Request) -> dict:
        """
        Get the request arguments based on the request headers and data.

        :param Request request: The request object.
        :return: The request arguments.
        :rtype: dict
        """
        headers = request.headers or {}
        data = request.body or {}
        content_type = headers.get("Content-Type", "application/json")

        if request.method == "GET" and not data:
            return {}

        if content_type.startswith("application/") and "json" in content_type:
            return {"json": data}

        if "multipart/form-data" in content_type:
            return {"files": data}

        return {"data": data}



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class AdminUserCreateRequest(BaseModel):
    """AdminUserCreateRequest

    :param first_name: User's first name
    :type first_name: str
    :param last_name: User's last name
    :type last_name: str
    :param email: User's email
    :type email: str
    """

    def __init__(self, first_name: str, last_name: str, email: str):
        self.first_name = first_name
        self.last_name = last_name
        self.email = email



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .eor_timeoffs_base_item import EorTimeoffsBaseItem


@JsonMap({})
class EorClientTimeoffRequests(BaseModel):
    """EorClientTimeoffRequests

    :param year: year, defaults to None
    :type year: float, optional
    :param days_requested: Days off requested by employee., defaults to None
    :type days_requested: float, optional
    :param current_approved: Days off approved., defaults to None
    :type current_approved: float, optional
    :param yearly_limit: Total number of days off allowed per year., defaults to None
    :type yearly_limit: float, optional
    :param over_annual_limit: Too many days off requested by employee., defaults to None
    :type over_annual_limit: bool, optional
    :param yearly_requests: Total number of time off requests this year., defaults to None
    :type yearly_requests: float, optional
    :param requests: List of time offs.
    :type requests: List[EorTimeoffsBaseItem]
    """

    def __init__(
        self,
        requests: List[EorTimeoffsBaseItem],
        year: float = None,
        days_requested: float = None,
        current_approved: float = None,
        yearly_limit: float = None,
        over_annual_limit: bool = None,
        yearly_requests: float = None,
    ):
        if year is not None:
            self.year = year
        if days_requested is not None:
            self.days_requested = days_requested
        if current_approved is not None:
            self.current_approved = current_approved
        if yearly_limit is not None:
            self.yearly_limit = yearly_limit
        if over_annual_limit is not None:
            self.over_annual_limit = over_annual_limit
        if yearly_requests is not None:
            self.yearly_requests = yearly_requests
        self.requests = self._define_list(requests, EorTimeoffsBaseItem)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class AdditionalEorInfoContainer(BaseModel):
    """AdditionalEorInfoContainer

    :param data: Fields for country validation mandatory found at `/eor/validations/{country_code}`.
    :type data: any
    """

    def __init__(self, data: any):
        self.data = data



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .contract_to_create_ongoing_time_based import ContractToCreateOngoingTimeBased


@JsonMap({})
class ContractToCreateContainerOngoingTimeBased(BaseModel):
    """ContractToCreateContainerOngoingTimeBased

    :param data: data
    :type data: ContractToCreateOngoingTimeBased
    """

    def __init__(self, data: ContractToCreateOngoingTimeBased):
        self.data = self._define_object(data, ContractToCreateOngoingTimeBased)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .contract_to_create import ContractToCreate, ContractToCreateGuard
from .contract_to_create_pay_as_you_go_time_based import (
    ContractToCreatePayAsYouGoTimeBased,
)
from .contract_to_create_payg_tasks import ContractToCreatePaygTasks
from .contract_to_create_payg_milestones import ContractToCreatePaygMilestones
from .contract_to_create_ongoing_time_based import ContractToCreateOngoingTimeBased


@JsonMap({})
class ContractToCreateContainer(BaseModel):
    """ContractToCreateContainer

    :param data: data
    :type data: ContractToCreate
    """

    def __init__(self, data: ContractToCreate):
        self.data = ContractToCreateGuard.return_one_of(data)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .state_of_country import StateOfCountry


@JsonMap({})
class Country(BaseModel):
    """Country

    :param code: Country code.
    :type code: str
    :param name: Country name.
    :type name: str
    :param visa_support: Identifies if Deel provides visa support for employees being hired in this country.
    :type visa_support: bool
    :param eor_support: Identifies if Deel support EoR in this country.
    :type eor_support: bool
    :param state_type: Identifies the type of sub-territory within a country where local laws may apply. Some countries may have states, provinces, prefectures or regions.
    :type state_type: str
    :param states: states
    :type states: List[StateOfCountry]
    """

    def __init__(
        self,
        code: str,
        name: str,
        visa_support: bool,
        eor_support: bool,
        state_type: str,
        states: List[StateOfCountry],
    ):
        self.code = self._pattern_matching(code, "^[A-Z]{2}$", "code")
        self.name = name
        self.visa_support = visa_support
        self.eor_support = eor_support
        self.state_type = state_type
        self.states = self._define_list(states, StateOfCountry)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class TimeoffReview(BaseModel):
    """TimeoffReview

    :param is_approved: Approve or decline employee's time off request.
    :type is_approved: bool
    :param denial_reason: Denial reason for the time off requested., defaults to None
    :type denial_reason: str, optional
    """

    def __init__(self, is_approved: bool, denial_reason: str = None):
        self.is_approved = is_approved
        if denial_reason is not None:
            self.denial_reason = denial_reason



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id"})
class AdminUser(BaseModel):
    """AdminUser

    :param id_: Unique identifier of this resource.
    :type id_: str
    :param first_name: User's first name
    :type first_name: str
    :param last_name: User's last name
    :type last_name: str
    :param email: User's email
    :type email: str
    """

    def __init__(self, id_: str, first_name: str, last_name: str, email: str):
        self.id_ = id_
        self.first_name = first_name
        self.last_name = last_name
        self.email = email



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .invoice_download_object import InvoiceDownloadObject


@JsonMap({})
class InvoiceDownloadLinkContainer(BaseModel):
    """InvoiceDownloadLinkContainer

    :param data: data
    :type data: InvoiceDownloadObject
    """

    def __init__(self, data: InvoiceDownloadObject):
        self.data = self._define_object(data, InvoiceDownloadObject)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .contract_to_create_payg_milestones import ContractToCreatePaygMilestones


@JsonMap({})
class ContractToCreateContainerPaygMilestones(BaseModel):
    """ContractToCreateContainerPaygMilestones

    :param data: data
    :type data: ContractToCreatePaygMilestones
    """

    def __init__(self, data: ContractToCreatePaygMilestones):
        self.data = self._define_object(data, ContractToCreatePaygMilestones)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .off_cycle_payment_to_create import OffCyclePaymentToCreate


@JsonMap({})
class OffCyclePaymentToCreateContainer(BaseModel):
    """OffCyclePaymentToCreateContainer

    :param data: Details of invoice adjustment to create.
    :type data: OffCyclePaymentToCreate
    """

    def __init__(self, data: OffCyclePaymentToCreate):
        self.data = self._define_object(data, OffCyclePaymentToCreate)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .contract_type_enum import ContractTypeEnum
from .contract_status_enum import ContractStatusEnum
from .seniority import Seniority


@JsonMap({})
class EorContractCreatedEmployment(BaseModel):
    """EorContractCreatedEmployment

    :param scope_of_work: Scope of work description., defaults to None
    :type scope_of_work: str, optional
    :param start_date: Short date in format ISO-8601 (YYYY-MM-DD). For example 2022-12-31., defaults to None
    :type start_date: str, optional
    :param end_date: Short date in format ISO-8601 (YYYY-MM-DD). For example 2022-12-31., defaults to None
    :type end_date: str, optional
    :param country: Country of employement., defaults to None
    :type country: str, optional
    :param state: State code of the state/province where the this person will be employed., defaults to None
    :type state: str, optional
    :param work_visa_required: Do you require Deel to apply for work visa for this person?, defaults to None
    :type work_visa_required: bool, optional
    :param time_off_type: If you want to use standard number of holidays for this employee, choose "STANDARD". If you want to enter a specific number of holidays, choose "SPECIFIC" and enter the number of days in the holidays field., defaults to None
    :type time_off_type: str, optional
    :param probation_period: Number of probation days., defaults to None
    :type probation_period: float, optional
    :param calculated_holidays: Number of calculate holidays for this contract., defaults to None
    :type calculated_holidays: str, optional
    """

    def __init__(
        self,
        scope_of_work: str = None,
        start_date: str = None,
        end_date: str = None,
        country: str = None,
        state: str = None,
        work_visa_required: bool = None,
        time_off_type: str = None,
        probation_period: float = None,
        calculated_holidays: str = None,
    ):
        if scope_of_work is not None:
            self.scope_of_work = scope_of_work
        if start_date is not None:
            self.start_date = self._pattern_matching(
                start_date, "^\d{4}-\d{2}-\d{2}$", "start_date"
            )
        if end_date is not None:
            self.end_date = self._pattern_matching(
                end_date, "^\d{4}-\d{2}-\d{2}$", "end_date"
            )
        if country is not None:
            self.country = country
        if state is not None:
            self.state = state
        if work_visa_required is not None:
            self.work_visa_required = work_visa_required
        if time_off_type is not None:
            self.time_off_type = time_off_type
        if probation_period is not None:
            self.probation_period = probation_period
        if calculated_holidays is not None:
            self.calculated_holidays = calculated_holidays


@JsonMap({})
class ClientLegalEntity2(BaseModel):
    """ClientLegalEntity2

    :param name: Legal entity name., defaults to None
    :type name: str, optional
    """

    def __init__(self, name: str = None):
        if name is not None:
            self.name = name


@JsonMap({})
class EorContractCreatedClient(BaseModel):
    """EorContractCreatedClient

    :param legal_entity: legal_entity, defaults to None
    :type legal_entity: ClientLegalEntity2, optional
    """

    def __init__(self, legal_entity: ClientLegalEntity2 = None):
        if legal_entity is not None:
            self.legal_entity = self._define_object(legal_entity, ClientLegalEntity2)


@JsonMap({})
class EorContractCreatedCompensationDetails(BaseModel):
    """EorContractCreatedCompensationDetails

    :param salary: Gross annual salary., defaults to None
    :type salary: float, optional
    :param currency: Currency of gross annual salary., defaults to None
    :type currency: str, optional
    :param variable_compensation: Variable compensation acount or percentage., defaults to None
    :type variable_compensation: str, optional
    :param variable_compensation_type: Type fo variable compensation., defaults to None
    :type variable_compensation_type: str, optional
    """

    def __init__(
        self,
        salary: float = None,
        currency: str = None,
        variable_compensation: str = None,
        variable_compensation_type: str = None,
    ):
        if salary is not None:
            self.salary = salary
        if currency is not None:
            self.currency = currency
        if variable_compensation is not None:
            self.variable_compensation = variable_compensation
        if variable_compensation_type is not None:
            self.variable_compensation_type = variable_compensation_type


@JsonMap({})
class EorContractCreatedEmployee(BaseModel):
    """EorContractCreatedEmployee

    :param legal_name: Employee's legal name., defaults to None
    :type legal_name: str, optional
    :param first_name: Employee's first name., defaults to None
    :type first_name: str, optional
    :param last_name: Employee's last name., defaults to None
    :type last_name: str, optional
    :param email: Employee's email., defaults to None
    :type email: str, optional
    """

    def __init__(
        self,
        legal_name: str = None,
        first_name: str = None,
        last_name: str = None,
        email: str = None,
    ):
        if legal_name is not None:
            self.legal_name = legal_name
        if first_name is not None:
            self.first_name = first_name
        if last_name is not None:
            self.last_name = last_name
        if email is not None:
            self.email = email


@JsonMap({"id_": "id"})
class HealthPlan(BaseModel):
    """HealthPlan

    :param id_: Health plan ID, defaults to None
    :type id_: str, optional
    :param name: Health plan name, defaults to None
    :type name: str, optional
    """

    def __init__(self, id_: str = None, name: str = None):
        if id_ is not None:
            self.id_ = id_
        if name is not None:
            self.name = name


@JsonMap({"id_": "id", "type_": "type"})
class EorContractCreated(BaseModel):
    """EorContractCreated

    :param id_: Id of the contract quote created, defaults to None
    :type id_: str, optional
    :param type_: Type of a contract, defaults to None
    :type type_: ContractTypeEnum, optional
    :param created_at: Long date-time format following ISO-8601, defaults to None
    :type created_at: str, optional
    :param status: Status of a contract in Deel workflow, defaults to None
    :type status: ContractStatusEnum, optional
    :param job_title: Employee's job title., defaults to None
    :type job_title: str, optional
    :param employment: employment, defaults to None
    :type employment: EorContractCreatedEmployment, optional
    :param client: client, defaults to None
    :type client: EorContractCreatedClient, optional
    :param compensation_details: compensation_details, defaults to None
    :type compensation_details: EorContractCreatedCompensationDetails, optional
    :param employee: employee, defaults to None
    :type employee: EorContractCreatedEmployee, optional
    :param health_plan: health_plan, defaults to None
    :type health_plan: HealthPlan, optional
    :param seniority: Seniority describes level of expertise at a job e.g. junior, defaults to None
    :type seniority: Seniority, optional
    """

    def __init__(
        self,
        id_: str = None,
        type_: ContractTypeEnum = None,
        created_at: str = None,
        status: ContractStatusEnum = None,
        job_title: str = None,
        employment: EorContractCreatedEmployment = None,
        client: EorContractCreatedClient = None,
        compensation_details: EorContractCreatedCompensationDetails = None,
        employee: EorContractCreatedEmployee = None,
        health_plan: HealthPlan = None,
        seniority: Seniority = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if type_ is not None:
            self.type_ = self._enum_matching(type_, ContractTypeEnum.list(), "type_")
        if created_at is not None:
            self.created_at = created_at
        if status is not None:
            self.status = self._enum_matching(
                status, ContractStatusEnum.list(), "status"
            )
        if job_title is not None:
            self.job_title = job_title
        if employment is not None:
            self.employment = self._define_object(
                employment, EorContractCreatedEmployment
            )
        if client is not None:
            self.client = self._define_object(client, EorContractCreatedClient)
        if compensation_details is not None:
            self.compensation_details = self._define_object(
                compensation_details, EorContractCreatedCompensationDetails
            )
        if employee is not None:
            self.employee = self._define_object(employee, EorContractCreatedEmployee)
        if health_plan is not None:
            self.health_plan = self._define_object(health_plan, HealthPlan)
        if seniority is not None:
            self.seniority = self._define_object(seniority, Seniority)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class MonthlyPayment(BaseModel):
    """Monthly payment details for the user

    :param count: The number of monthly payments, defaults to None
    :type count: int, optional
    """

    def __init__(self, count: int = None):
        if count is not None:
            self.count = count



# This file was generated by liblab | https://liblab.com/

from enum import Enum
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel


class WebhookItemStatus(Enum):
    """An enumeration representing different categories.

    :cvar ENABLED: "enabled"
    :vartype ENABLED: str
    :cvar DISABLED: "disabled"
    :vartype DISABLED: str
    """

    ENABLED = "enabled"
    DISABLED = "disabled"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, WebhookItemStatus._member_map_.values()))


@JsonMap({"id_": "id"})
class WebhookItem(BaseModel):
    """WebhookItem

    :param id_: Unique identifier of this resource.
    :type id_: str
    :param name: Webhook subscription name.
    :type name: str
    :param description: Webhook subscription description.
    :type description: str
    :param status: Status of webhook.
    :type status: WebhookItemStatus
    :param url: Endpoint to receive webhook.
    :type url: str
    :param signing_key: The webhook's signing key, used to generate webhook signatures.
    :type signing_key: str
    :param api_version: Deel API version. Currently Deel accepts v1 or v2 version.
    :type api_version: str
    :param events: The list of events to enable for this subscription.
    :type events: List[str]
    :param created_at: Time at which the webhook was created.
    :type created_at: str
    :param updated_at: Time at which the webhook was updated.
    :type updated_at: str
    :param hidden: Property to hide the webhook from frontend view. Commonly by Dell 3rd party apps, defaults to None
    :type hidden: bool, optional
    :param internal: internal, defaults to None
    :type internal: bool, optional
    :param deleted_at: Time at which the webhook was deleted., defaults to None
    :type deleted_at: str, optional
    """

    def __init__(
        self,
        id_: str,
        name: str,
        description: str,
        status: WebhookItemStatus,
        url: str,
        signing_key: str,
        api_version: str,
        events: List[str],
        created_at: str,
        updated_at: str,
        hidden: bool = None,
        internal: bool = None,
        deleted_at: str = None,
    ):
        self.id_ = id_
        self.name = name
        self.description = description
        self.status = self._enum_matching(status, WebhookItemStatus.list(), "status")
        self.url = url
        self.signing_key = signing_key
        self.api_version = api_version
        self.events = events
        self.created_at = created_at
        self.updated_at = updated_at
        if hidden is not None:
            self.hidden = hidden
        if internal is not None:
            self.internal = internal
        if deleted_at is not None:
            self.deleted_at = deleted_at



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id"})
class GpClientLegalEntity(BaseModel):
    """GpClientLegalEntity

    :param id_: Unique identifier of this resource.
    :type id_: str
    """

    def __init__(self, id_: str):
        self.id_ = id_


@JsonMap({"id_": "id"})
class GpClientTeam(BaseModel):
    """GpClientTeam

    :param id_: Unique identifier of this resource.
    :type id_: str
    """

    def __init__(self, id_: str):
        self.id_ = id_


@JsonMap({})
class GpClient(BaseModel):
    """GpClient

    :param legal_entity: legal_entity
    :type legal_entity: GpClientLegalEntity
    :param team: team
    :type team: GpClientTeam
    """

    def __init__(self, legal_entity: GpClientLegalEntity, team: GpClientTeam):
        self.legal_entity = self._define_object(legal_entity, GpClientLegalEntity)
        self.team = self._define_object(team, GpClientTeam)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class GpEmployeePtoUpdateData(BaseModel):
    """GpEmployeePtoUpdateData

    :param accrual_start_date: Short date in format ISO-8601 (YYYY-MM-DD). For example: 2022-12-31.
    :type accrual_start_date: str
    :param yearly_allowance: Enter the number of holidays.
    :type yearly_allowance: str
    """

    def __init__(self, accrual_start_date: str, yearly_allowance: str):
        self.accrual_start_date = accrual_start_date
        self.yearly_allowance = yearly_allowance



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class ContractStatusEnum(Enum):
    """An enumeration representing different categories.

    :cvar NEW: "new"
    :vartype NEW: str
    :cvar UNDER_REVIEW: "under_review"
    :vartype UNDER_REVIEW: str
    :cvar WAITING_FOR_EMPLOYEE_CONTRACT: "waiting_for_employee_contract"
    :vartype WAITING_FOR_EMPLOYEE_CONTRACT: str
    :cvar WAITING_FOR_CLIENT_SIGN: "waiting_for_client_sign"
    :vartype WAITING_FOR_CLIENT_SIGN: str
    :cvar PROCESSING_PAYMENT: "processing_payment"
    :vartype PROCESSING_PAYMENT: str
    :cvar WAITING_FOR_CONTRACTOR_SIGN: "waiting_for_contractor_sign"
    :vartype WAITING_FOR_CONTRACTOR_SIGN: str
    :cvar WAITING_FOR_EOR_SIGN: "waiting_for_eor_sign"
    :vartype WAITING_FOR_EOR_SIGN: str
    :cvar WAITING_FOR_EMPLOYEE_SIGN: "waiting_for_employee_sign"
    :vartype WAITING_FOR_EMPLOYEE_SIGN: str
    :cvar AWAITING_DEPOSIT_PAYMENT: "awaiting_deposit_payment"
    :vartype AWAITING_DEPOSIT_PAYMENT: str
    :cvar IN_PROGRESS: "in_progress"
    :vartype IN_PROGRESS: str
    :cvar COMPLETED: "completed"
    :vartype COMPLETED: str
    :cvar CANCELLED: "cancelled"
    :vartype CANCELLED: str
    :cvar USER_CANCELLED: "user_cancelled"
    :vartype USER_CANCELLED: str
    :cvar REJECTED: "rejected"
    :vartype REJECTED: str
    :cvar WAITING_FOR_CLIENT_PAYMENT: "waiting_for_client_payment"
    :vartype WAITING_FOR_CLIENT_PAYMENT: str
    :cvar ONBOARDING: "onboarding"
    :vartype ONBOARDING: str
    """

    NEW = "new"
    UNDER_REVIEW = "under_review"
    WAITING_FOR_EMPLOYEE_CONTRACT = "waiting_for_employee_contract"
    WAITING_FOR_CLIENT_SIGN = "waiting_for_client_sign"
    PROCESSING_PAYMENT = "processing_payment"
    WAITING_FOR_CONTRACTOR_SIGN = "waiting_for_contractor_sign"
    WAITING_FOR_EOR_SIGN = "waiting_for_eor_sign"
    WAITING_FOR_EMPLOYEE_SIGN = "waiting_for_employee_sign"
    AWAITING_DEPOSIT_PAYMENT = "awaiting_deposit_payment"
    IN_PROGRESS = "in_progress"
    COMPLETED = "completed"
    CANCELLED = "cancelled"
    USER_CANCELLED = "user_cancelled"
    REJECTED = "rejected"
    WAITING_FOR_CLIENT_PAYMENT = "waiting_for_client_payment"
    ONBOARDING = "onboarding"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ContractStatusEnum._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .base import BaseModel
from .week_days_enum import WeekDaysEnum


class ProRataCalculationType(Enum):
    """An enumeration representing different categories.

    :cvar CUSTOM_AMOUNT: "CUSTOM_AMOUNT"
    :vartype CUSTOM_AMOUNT: str
    :cvar WORK_DAYS: "WORK_DAYS"
    :vartype WORK_DAYS: str
    :cvar CALENDAR_DAYS: "CALENDAR_DAYS"
    :vartype CALENDAR_DAYS: str
    :cvar FULL_AMOUNT: "FULL_AMOUNT"
    :vartype FULL_AMOUNT: str
    """

    CUSTOM_AMOUNT = "CUSTOM_AMOUNT"
    WORK_DAYS = "WORK_DAYS"
    CALENDAR_DAYS = "CALENDAR_DAYS"
    FULL_AMOUNT = "FULL_AMOUNT"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, ProRataCalculationType._member_map_.values())
        )


@JsonMap({})
class ProRata(BaseModel):
    """Details of the pro rata for the first payment

    :param calculation_type: Either works days or calendar days, defaults to None
    :type calculation_type: ProRataCalculationType, optional
    :param rate: Rate to be paid., defaults to None
    :type rate: float, optional
    :param daily_rate: Daily rate to be paid., defaults to None
    :type daily_rate: float, optional
    :param total: Total to be paid., defaults to None
    :type total: float, optional
    :param cycle_work_days: Total days to be work., defaults to None
    :type cycle_work_days: float, optional
    :param work_week_start: Days of the week., defaults to None
    :type work_week_start: WeekDaysEnum, optional
    :param work_week_end: Days of the week., defaults to None
    :type work_week_end: WeekDaysEnum, optional
    :param cycle_start: Short date in format ISO-8601 (YYYY-MM-DD). For example: 2022-12-31., defaults to None
    :type cycle_start: str, optional
    :param cycle_end: Short date in format ISO-8601 (YYYY-MM-DD). For example: 2022-12-31., defaults to None
    :type cycle_end: str, optional
    """

    def __init__(
        self,
        calculation_type: ProRataCalculationType = None,
        rate: float = None,
        daily_rate: float = None,
        total: float = None,
        cycle_work_days: float = None,
        work_week_start: WeekDaysEnum = None,
        work_week_end: WeekDaysEnum = None,
        cycle_start: str = None,
        cycle_end: str = None,
    ):
        if calculation_type is not None:
            self.calculation_type = self._enum_matching(
                calculation_type, ProRataCalculationType.list(), "calculation_type"
            )
        if rate is not None:
            self.rate = rate
        if daily_rate is not None:
            self.daily_rate = daily_rate
        if total is not None:
            self.total = total
        if cycle_work_days is not None:
            self.cycle_work_days = cycle_work_days
        if work_week_start is not None:
            self.work_week_start = self._enum_matching(
                work_week_start, WeekDaysEnum.list(), "work_week_start"
            )
        if work_week_end is not None:
            self.work_week_end = self._enum_matching(
                work_week_end, WeekDaysEnum.list(), "work_week_end"
            )
        if cycle_start is not None:
            self.cycle_start = cycle_start
        if cycle_end is not None:
            self.cycle_end = cycle_end



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .pro_rata import ProRata
from .first_payment_date import FirstPaymentDate


@JsonMap({})
class ResponseEstimateFirstPayment(BaseModel):
    """ResponseEstimateFirstPayment

    :param pro_rata: Details of the pro rata for the first payment, defaults to None
    :type pro_rata: ProRata, optional
    :param first_payment_dates: first_payment_dates, defaults to None
    :type first_payment_dates: List[FirstPaymentDate], optional
    """

    def __init__(
        self,
        pro_rata: ProRata = None,
        first_payment_dates: List[FirstPaymentDate] = None,
    ):
        if pro_rata is not None:
            self.pro_rata = self._define_object(pro_rata, ProRata)
        if first_payment_dates is not None:
            self.first_payment_dates = self._define_list(
                first_payment_dates, FirstPaymentDate
            )



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class TimesheetToUpdate(BaseModel):
    """Details of the timesheet to update. Both a client or a contractor may update a timesheet.

    :param description: description
    :type description: str
    :param quantity: quantity
    :type quantity: float
    """

    def __init__(self, description: str, quantity: float):
        self.description = description
        self.quantity = quantity



# This file was generated by liblab | https://liblab.com/

from enum import Enum
from .utils.json_map import JsonMap
from .base import BaseModel


class HrisContractPartTimeEmploymentType(Enum):
    """An enumeration representing different categories.

    :cvar PART_TIME: "PART_TIME"
    :vartype PART_TIME: str
    """

    PART_TIME = "PART_TIME"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                HrisContractPartTimeEmploymentType._member_map_.values(),
            )
        )


@JsonMap({})
class HrisContractPartTime(BaseModel):
    """HrisContractPartTime

    :param contract_oid: The Hris Direct employee contract ID
    :type contract_oid: str
    :param start_date: Short date in format ISO-8601 (YYYY-MM-DD). For example: 2022-12-31.
    :type start_date: str
    :param employee_number: employee_number, defaults to None
    :type employee_number: float, optional
    :param end_date: Short date in format ISO-8601 (YYYY-MM-DD). For example 2022-12-31., defaults to None
    :type end_date: str, optional
    :param employment_type: employment_type
    :type employment_type: HrisContractPartTimeEmploymentType
    :param part_time_percentage: required if employmentType is "PART_TIME"
    :type part_time_percentage: float
    """

    def __init__(
        self,
        contract_oid: str,
        start_date: str,
        employment_type: HrisContractPartTimeEmploymentType,
        part_time_percentage: float,
        employee_number: float = None,
        end_date: str = None,
    ):
        self.contract_oid = contract_oid
        self.start_date = start_date
        if employee_number is not None:
            self.employee_number = employee_number
        if end_date is not None:
            self.end_date = self._pattern_matching(
                end_date, "^\d{4}-\d{2}-\d{2}$", "end_date"
            )
        self.employment_type = self._enum_matching(
            employment_type,
            HrisContractPartTimeEmploymentType.list(),
            "employment_type",
        )
        self.part_time_percentage = part_time_percentage



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class BenefitContributionTypeEnum(Enum):
    """An enumeration representing different categories.

    :cvar PERCENTAGE: "PERCENTAGE"
    :vartype PERCENTAGE: str
    :cvar FIXED_AMOUNT: "FIXED_AMOUNT"
    :vartype FIXED_AMOUNT: str
    :cvar MATCH: "MATCH"
    :vartype MATCH: str
    """

    PERCENTAGE = "PERCENTAGE"
    FIXED_AMOUNT = "FIXED_AMOUNT"
    MATCH = "MATCH"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, BenefitContributionTypeEnum._member_map_.values())
        )



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class TimesheetToCreateContainerWithFile(BaseModel):
    """TimesheetToCreateContainerWithFile

    :param contract_id: Id of a Deel contract.
    :type contract_id: str
    :param description: description
    :type description: str
    :param date_submitted: Short date in format ISO-8601 (YYYY-MM-DD). For example: 2022-12-31.
    :type date_submitted: str
    :param quantity: This is based on the scale of work statement of the associated contract
    :type quantity: float
    :param file: Upload the file you want to attach to this entry., defaults to None
    :type file: str, optional
    """

    def __init__(
        self,
        contract_id: str,
        description: str,
        date_submitted: str,
        quantity: float,
        file: str = None,
    ):
        self.contract_id = contract_id
        self.description = description
        self.date_submitted = date_submitted
        self.quantity = quantity
        if file is not None:
            self.file = file



# This file was generated by liblab | https://liblab.com/

from enum import Enum
from typing import Any
from ...models.base import BaseModel


def extract_original_data(data: Any) -> Any:
    """
    Extracts the original data from internal models and enums.

    :param Any data: The data to be extracted.
    :return: The extracted data.
    :rtype: Any
    """
    if data is None:
        return None

    data_type = type(data)

    if issubclass(data_type, BaseModel):
        return data._map()

    if issubclass(data_type, Enum):
        return data.value

    if issubclass(data_type, list):
        return [extract_original_data(item) for item in data]

    return data



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id"})
class WebhookEventTypeResponse(BaseModel):
    """WebhookEventTypeResponse

    :param id_: Event type unique identifier.
    :type id_: float
    :param module_name: Name of the rabbit queue.
    :type module_name: str
    :param module_label: Display name of the rabbit queue in Deel UI.
    :type module_label: str
    :param name: Name of the webhook event.
    :type name: str
    :param description: Describes the webhook event and other pertinent info.
    :type description: str
    :param payload_example: JSON payload example of the specific event., defaults to None
    :type payload_example: str, optional
    :param created_at: Time at which the event type was created., defaults to None
    :type created_at: str, optional
    :param updated_at: Time at which the event type was updated., defaults to None
    :type updated_at: str, optional
    """

    def __init__(
        self,
        id_: float,
        module_name: str,
        module_label: str,
        name: str,
        description: str,
        payload_example: str = None,
        created_at: str = None,
        updated_at: str = None,
    ):
        self.id_ = id_
        self.module_name = module_name
        self.module_label = module_label
        self.name = name
        self.description = description
        if payload_example is not None:
            self.payload_example = payload_example
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .identifier_value import IdentifierValue, IdentifierValueGuard


@JsonMap({"id_": "id"})
class TimeoffsAttachmentsItem(BaseModel):
    """A item in attachments array. 2022-09-08 FE only allows for "SICK LEAVE"

    :param id_: id_, defaults to None
    :type id_: IdentifierValue, optional
    :param label: The file name, defaults to None
    :type label: str, optional
    :param file_extension: The file name extension, defaults to None
    :type file_extension: str, optional
    """

    def __init__(
        self, id_: IdentifierValue = None, label: str = None, file_extension: str = None
    ):
        if id_ is not None:
            self.id_ = IdentifierValueGuard.return_one_of(id_)
        if label is not None:
            self.label = label
        if file_extension is not None:
            self.file_extension = file_extension



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class AdjustmentStatusEnum(Enum):
    """An enumeration representing different categories.

    :cvar OPEN: "open"
    :vartype OPEN: str
    :cvar PENDING_APPROVAL: "pending_approval"
    :vartype PENDING_APPROVAL: str
    :cvar OVERWRITTEN: "overwritten"
    :vartype OVERWRITTEN: str
    :cvar FAILED: "failed"
    :vartype FAILED: str
    :cvar SUCCESS: "success"
    :vartype SUCCESS: str
    """

    OPEN = "open"
    PENDING_APPROVAL = "pending_approval"
    OVERWRITTEN = "overwritten"
    FAILED = "failed"
    SUCCESS = "success"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, AdjustmentStatusEnum._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id"})
class WorkerTermination(BaseModel):
    """WorkerTermination

    :param id_: id_
    :type id_: str
    :param last_date_of_work: Long date-time format following ISO-8601
    :type last_date_of_work: str
    """

    def __init__(self, id_: str, last_date_of_work: str):
        self.id_ = id_
        self.last_date_of_work = last_date_of_work



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .cost_quote import CostQuote


@JsonMap({})
class Monthly(BaseModel):
    """Monthly

    :param costs: costs, defaults to None
    :type costs: List[CostQuote], optional
    :param total: Total costs in local currency, defaults to None
    :type total: float, optional
    :param total_usd: Total costs in USD, defaults to None
    :type total_usd: float, optional
    """

    def __init__(
        self,
        costs: List[CostQuote] = None,
        total: float = None,
        total_usd: float = None,
    ):
        if costs is not None:
            self.costs = self._define_list(costs, CostQuote)
        if total is not None:
            self.total = total
        if total_usd is not None:
            self.total_usd = total_usd


@JsonMap({})
class BreakdownCostsQuote(BaseModel):
    """BreakdownCostsQuote

    :param monthly: monthly, defaults to None
    :type monthly: Monthly, optional
    """

    def __init__(self, monthly: Monthly = None):
        if monthly is not None:
            self.monthly = self._define_object(monthly, Monthly)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .employee_timeoffs_item_response import EmployeeTimeoffsItemResponse


@JsonMap({})
class EmployeeTimeoffsCreationContainer(BaseModel):
    """EmployeeTimeoffsCreationContainer

    :param data: data
    :type data: EmployeeTimeoffsItemResponse
    """

    def __init__(self, data: EmployeeTimeoffsItemResponse):
        self.data = self._define_object(data, EmployeeTimeoffsItemResponse)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .deel_invoice import DeelInvoice
from .page_info_without_cursor_new import PageInfoWithoutCursorNew


@JsonMap({})
class DeelInvoiceListContainer(BaseModel):
    """DeelInvoiceListContainer

    :param data: data
    :type data: List[DeelInvoice]
    :param page: page
    :type page: PageInfoWithoutCursorNew
    """

    def __init__(self, data: List[DeelInvoice], page: PageInfoWithoutCursorNew):
        self.data = self._define_list(data, DeelInvoice)
        self.page = self._define_object(page, PageInfoWithoutCursorNew)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class GenericResultCreatedData(BaseModel):
    """GenericResultCreatedData

    :param created: created
    :type created: bool
    """

    def __init__(self, created: bool):
        self.created = created


@JsonMap({})
class GenericResultCreated(BaseModel):
    """GenericResultCreated

    :param data: data
    :type data: GenericResultCreatedData
    """

    def __init__(self, data: GenericResultCreatedData):
        self.data = self._define_object(data, GenericResultCreatedData)



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class PaymentMethodEnum(Enum):
    """An enumeration representing different categories.

    :cvar STRIPE_BACS_DEBIT: "stripe_bacs_debit"
    :vartype STRIPE_BACS_DEBIT: str
    :cvar PAY_PAL: "pay_pal"
    :vartype PAY_PAL: str
    :cvar STRIPE_CARD: "stripe_card"
    :vartype STRIPE_CARD: str
    :cvar BT_CARD: "bt_card"
    :vartype BT_CARD: str
    :cvar STRIPE_ACH: "stripe_ach"
    :vartype STRIPE_ACH: str
    :cvar COINBASE: "coinbase"
    :vartype COINBASE: str
    :cvar GO_CARDLESS: "go_cardless"
    :vartype GO_CARDLESS: str
    :cvar BREX: "brex"
    :vartype BREX: str
    :cvar STRIPE_SEPA_DEBIT: "stripe_sepa_debit"
    :vartype STRIPE_SEPA_DEBIT: str
    :cvar TRANSFERWISE: "transferwise"
    :vartype TRANSFERWISE: str
    :cvar GO_CARDLESS_BECS: "go_cardless_becs"
    :vartype GO_CARDLESS_BECS: str
    :cvar BANK_TRANSFER: "bank_transfer"
    :vartype BANK_TRANSFER: str
    :cvar MERCURY_WIRE: "mercury_wire"
    :vartype MERCURY_WIRE: str
    :cvar BT_PAY_PAL: "bt_pay_pal"
    :vartype BT_PAY_PAL: str
    """

    STRIPE_BACS_DEBIT = "stripe_bacs_debit"
    PAY_PAL = "pay_pal"
    STRIPE_CARD = "stripe_card"
    BT_CARD = "bt_card"
    STRIPE_ACH = "stripe_ach"
    COINBASE = "coinbase"
    GO_CARDLESS = "go_cardless"
    BREX = "brex"
    STRIPE_SEPA_DEBIT = "stripe_sepa_debit"
    TRANSFERWISE = "transferwise"
    GO_CARDLESS_BECS = "go_cardless_becs"
    BANK_TRANSFER = "bank_transfer"
    MERCURY_WIRE = "mercury_wire"
    BT_PAY_PAL = "bt_pay_pal"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, PaymentMethodEnum._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .worker_termination_body import WorkerTerminationBody


@JsonMap({})
class WorkerTerminationBodyContainer(BaseModel):
    """WorkerTerminationBodyContainer

    :param data: data
    :type data: WorkerTerminationBody
    """

    def __init__(self, data: WorkerTerminationBody):
        self.data = self._define_object(data, WorkerTerminationBody)



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.milestone_to_create_container import MilestoneToCreateContainer
from ..models.milestone_reviews_to_create_container import (
    MilestoneReviewsToCreateContainer,
)
from ..models.milestone_review_to_create_container import (
    MilestoneReviewToCreateContainer,
)
from ..models.milestone_list_container import MilestoneListContainer
from ..models.milestone_container import MilestoneContainer
from ..models.generic_result_deleted import GenericResultDeleted
from ..models.generic_result_created import GenericResultCreated


class MilestonesService(BaseService):

    @cast_models
    def get_milestones_by_contract(self, contract_id: str) -> MilestoneListContainer:
        """Retrieve a list of milestones found for a contract.

        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: MilestoneListContainer
        """

        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/milestones",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return MilestoneListContainer._unmap(response)

    @cast_models
    def create_milestone(
        self, contract_id: str, request_body: MilestoneToCreateContainer = None
    ) -> GenericResultCreated:
        """Add a new milestone to contract.

        :param request_body: The request body., defaults to None
        :type request_body: MilestoneToCreateContainer, optional
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultCreated
        """

        Validator(MilestoneToCreateContainer).is_optional().validate(request_body)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/milestones",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultCreated._unmap(response)

    @cast_models
    def get_milestones_by_contract_and_id(
        self, contract_id: str, milestone_id: str
    ) -> MilestoneContainer:
        """Retrieve a single milestone.

        :param contract_id: Deel contract id.
        :type contract_id: str
        :param milestone_id: ID of milestone to return
        :type milestone_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: MilestoneContainer
        """

        Validator(str).validate(contract_id)
        Validator(str).validate(milestone_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/milestones/{{milestone_id}}",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .add_path("milestone_id", milestone_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return MilestoneContainer._unmap(response)

    @cast_models
    def delete_milestone_by_id(
        self, contract_id: str, milestone_id: str
    ) -> GenericResultDeleted:
        """Delete a single milestone from a contract.

        :param contract_id: Deel contract id.
        :type contract_id: str
        :param milestone_id: ID of milestone to return
        :type milestone_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultDeleted
        """

        Validator(str).validate(contract_id)
        Validator(str).validate(milestone_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/milestones/{{milestone_id}}",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .add_path("milestone_id", milestone_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)

        return GenericResultDeleted._unmap(response)

    @cast_models
    def create_milestone_review(
        self,
        contract_id: str,
        milestone_id: str,
        request_body: MilestoneReviewToCreateContainer = None,
    ) -> GenericResultCreated:
        """Review a milestone to approve or decline submitted work.

        :param request_body: The request body., defaults to None
        :type request_body: MilestoneReviewToCreateContainer, optional
        :param contract_id: Deel contract id.
        :type contract_id: str
        :param milestone_id: ID of milestone to return
        :type milestone_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultCreated
        """

        Validator(MilestoneReviewToCreateContainer).is_optional().validate(request_body)
        Validator(str).validate(contract_id)
        Validator(str).validate(milestone_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/milestones/{{milestone_id}}/reviews",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .add_path("milestone_id", milestone_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultCreated._unmap(response)

    @cast_models
    def create_milestone_reviews(
        self, contract_id: str, request_body: MilestoneReviewsToCreateContainer = None
    ) -> GenericResultCreated:
        """Review a batch of milestones to approve or reject submitted work.

        :param request_body: The request body., defaults to None
        :type request_body: MilestoneReviewsToCreateContainer, optional
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultCreated
        """

        Validator(MilestoneReviewsToCreateContainer).is_optional().validate(
            request_body
        )
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/milestones/many/reviews",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultCreated._unmap(response)



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class TimeoffsStatusEnum(Enum):
    """An enumeration representing different categories.

    :cvar REQUESTED: "REQUESTED"
    :vartype REQUESTED: str
    :cvar APPROVED: "APPROVED"
    :vartype APPROVED: str
    :cvar REJECTED: "REJECTED"
    :vartype REJECTED: str
    """

    REQUESTED = "REQUESTED"
    APPROVED = "APPROVED"
    REJECTED = "REJECTED"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, TimeoffsStatusEnum._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .breakdown_costs_quote import BreakdownCostsQuote


@JsonMap({})
class EorQuoteBase(BaseModel):
    """EOR quote approved by Deel

    :param annual_salary: Annual Salary in local currency, defaults to None
    :type annual_salary: float, optional
    :param currency: currency, defaults to None
    :type currency: str, optional
    :param breakdown_costs: breakdown_costs, defaults to None
    :type breakdown_costs: BreakdownCostsQuote, optional
    :param severance_accrual_total: Total severance accrual in local currency, defaults to None
    :type severance_accrual_total: float, optional
    :param severance_accrual_total_usd: Total severance accrual in USD, defaults to None
    :type severance_accrual_total_usd: float, optional
    :param recurring_allowance_total: Total recurring allowance in local currency, defaults to None
    :type recurring_allowance_total: float, optional
    :param recurring_allowance_total_usd: Total recurring allowance in USD, defaults to None
    :type recurring_allowance_total_usd: float, optional
    :param salary_total: Total salary in local currency, defaults to None
    :type salary_total: float, optional
    :param salary_total_usd: Total salary in USD, defaults to None
    :type salary_total_usd: float, optional
    :param employer_cost_total: Total employer cost in local currency, defaults to None
    :type employer_cost_total: float, optional
    :param employer_cost_total_usd: Total employer cost in USD, defaults to None
    :type employer_cost_total_usd: float, optional
    :param platform_fee_total: Total platform fee in local currency, defaults to None
    :type platform_fee_total: float, optional
    :param platform_fee_total_usd: Total platform fee in USD, defaults to None
    :type platform_fee_total_usd: float, optional
    :param mobility_fee_total: Total mobility fee in local currency, defaults to None
    :type mobility_fee_total: float, optional
    :param mobility_fee_total_usd: Total mobility fee in USD, defaults to None
    :type mobility_fee_total_usd: float, optional
    :param monthly_cost_total: Total monthly cost in local currency, defaults to None
    :type monthly_cost_total: float, optional
    :param monthly_cost_total_usd: Total monthly cost in USD, defaults to None
    :type monthly_cost_total_usd: float, optional
    :param gross_salary_total: Total gross salary in local currency, defaults to None
    :type gross_salary_total: float, optional
    :param gross_salary_total_usd: Total gross salary in USD, defaults to None
    :type gross_salary_total_usd: float, optional
    :param cba_total: Total CBA in local currency, defaults to None
    :type cba_total: float, optional
    :param cba_total_usd: Total CBA in USD, defaults to None
    :type cba_total_usd: float, optional
    :param created_at: Short date in format ISO-8601 (YYYY-MM-DD). For example 2022-12-31., defaults to None
    :type created_at: str, optional
    :param updated_at: Short date in format ISO-8601 (YYYY-MM-DD). For example 2022-12-31., defaults to None
    :type updated_at: str, optional
    """

    def __init__(
        self,
        annual_salary: float = None,
        currency: str = None,
        breakdown_costs: BreakdownCostsQuote = None,
        severance_accrual_total: float = None,
        severance_accrual_total_usd: float = None,
        recurring_allowance_total: float = None,
        recurring_allowance_total_usd: float = None,
        salary_total: float = None,
        salary_total_usd: float = None,
        employer_cost_total: float = None,
        employer_cost_total_usd: float = None,
        platform_fee_total: float = None,
        platform_fee_total_usd: float = None,
        mobility_fee_total: float = None,
        mobility_fee_total_usd: float = None,
        monthly_cost_total: float = None,
        monthly_cost_total_usd: float = None,
        gross_salary_total: float = None,
        gross_salary_total_usd: float = None,
        cba_total: float = None,
        cba_total_usd: float = None,
        created_at: str = None,
        updated_at: str = None,
    ):
        if annual_salary is not None:
            self.annual_salary = annual_salary
        if currency is not None:
            self.currency = self._pattern_matching(currency, "^[A-Z]{3}$", "currency")
        if breakdown_costs is not None:
            self.breakdown_costs = self._define_object(
                breakdown_costs, BreakdownCostsQuote
            )
        if severance_accrual_total is not None:
            self.severance_accrual_total = severance_accrual_total
        if severance_accrual_total_usd is not None:
            self.severance_accrual_total_usd = severance_accrual_total_usd
        if recurring_allowance_total is not None:
            self.recurring_allowance_total = recurring_allowance_total
        if recurring_allowance_total_usd is not None:
            self.recurring_allowance_total_usd = recurring_allowance_total_usd
        if salary_total is not None:
            self.salary_total = salary_total
        if salary_total_usd is not None:
            self.salary_total_usd = salary_total_usd
        if employer_cost_total is not None:
            self.employer_cost_total = employer_cost_total
        if employer_cost_total_usd is not None:
            self.employer_cost_total_usd = employer_cost_total_usd
        if platform_fee_total is not None:
            self.platform_fee_total = platform_fee_total
        if platform_fee_total_usd is not None:
            self.platform_fee_total_usd = platform_fee_total_usd
        if mobility_fee_total is not None:
            self.mobility_fee_total = mobility_fee_total
        if mobility_fee_total_usd is not None:
            self.mobility_fee_total_usd = mobility_fee_total_usd
        if monthly_cost_total is not None:
            self.monthly_cost_total = monthly_cost_total
        if monthly_cost_total_usd is not None:
            self.monthly_cost_total_usd = monthly_cost_total_usd
        if gross_salary_total is not None:
            self.gross_salary_total = gross_salary_total
        if gross_salary_total_usd is not None:
            self.gross_salary_total_usd = gross_salary_total_usd
        if cba_total is not None:
            self.cba_total = cba_total
        if cba_total_usd is not None:
            self.cba_total_usd = cba_total_usd
        if created_at is not None:
            self.created_at = self._pattern_matching(
                created_at, "^\d{4}-\d{2}-\d{2}$", "created_at"
            )
        if updated_at is not None:
            self.updated_at = self._pattern_matching(
                updated_at, "^\d{4}-\d{2}-\d{2}$", "updated_at"
            )



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class ContractTypeEnumForEstimate(Enum):
    """An enumeration representing different categories.

    :cvar ONGOING_TIME_BASED: "ongoing_time_based"
    :vartype ONGOING_TIME_BASED: str
    :cvar PAYG_TASKS: "payg_tasks"
    :vartype PAYG_TASKS: str
    """

    ONGOING_TIME_BASED = "ongoing_time_based"
    PAYG_TASKS = "payg_tasks"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, ContractTypeEnumForEstimate._member_map_.values())
        )



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .invoice_adjustment_to_update import InvoiceAdjustmentToUpdate


@JsonMap({})
class InvoiceAdjustmentToUpdateContainer(BaseModel):
    """InvoiceAdjustmentToUpdateContainer

    :param data: Details of invoice adjustment to update.
    :type data: InvoiceAdjustmentToUpdate
    """

    def __init__(self, data: InvoiceAdjustmentToUpdate):
        self.data = self._define_object(data, InvoiceAdjustmentToUpdate)



# This file was generated by liblab | https://liblab.com/

from typing import List
from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.worker_documents_by_id_container import WorkerDocumentsByIdContainer
from ..models.utils.cast_models import cast_models
from ..models.sort_dir_enum import SortDirEnum
from ..models.response_estimate_first_payment_container import (
    ResponseEstimateFirstPaymentContainer,
)
from ..models.input_to_patch_contract_external_id import InputToPatchContractExternalId
from ..models.get_contract_list_currencies import GetContractListCurrencies
from ..models.generic_result_updated import GenericResultUpdated
from ..models.generic_result_deleted import GenericResultDeleted
from ..models.generic_result_created import GenericResultCreated
from ..models.final_payment_calculated_container import FinalPaymentCalculatedContainer
from ..models.file_object import FileObject
from ..models.estimate_first_payment_container import EstimateFirstPaymentContainer
from ..models.download_worker_documents_by_id_container import (
    DownloadWorkerDocumentsByIdContainer,
)
from ..models.contracts_sort_by_enum import ContractsSortByEnum
from ..models.contract_type_enum import ContractTypeEnum
from ..models.contract_template_list_container import ContractTemplateListContainer
from ..models.contract_status_enum import ContractStatusEnum
from ..models.contract_signature_to_create_container import (
    ContractSignatureToCreateContainer,
)
from ..models.contract_list_container import ContractListContainer
from ..models.contract_invitation_to_create_container import (
    ContractInvitationToCreateContainer,
)
from ..models.contract_document_container import ContractDocumentContainer
from ..models.contract_container import ContractContainer
from ..models.calculate_final_payment_calculation_type import (
    CalculateFinalPaymentCalculationType,
)
from ..models.alternate_email_item import AlternateEmailItem


class ContractsService(BaseService):

    @cast_models
    def get_contract_list(
        self,
        after_cursor: str = None,
        limit: float = None,
        order_direction: SortDirEnum = None,
        types: List[ContractTypeEnum] = None,
        statuses: List[ContractStatusEnum] = None,
        team_id: str = None,
        external_id: str = None,
        countries: List[str] = None,
        currencies: GetContractListCurrencies = None,
        search: str = None,
        sort_by: ContractsSortByEnum = None,
    ) -> ContractListContainer:
        """Retrieve a list of contracts.

        :param after_cursor: Return next page of results after given cursor., defaults to None
        :type after_cursor: str, optional
        :param limit: Return a page of results with given number of records; NOTE technically ALL query parameters are strings or array of strings, defaults to None
        :type limit: float, optional
        :param order_direction: Order direction of results; ascending or descending., defaults to None
        :type order_direction: SortDirEnum, optional
        :param types: You can filter contracts by type, a contract is included in results if its type is in this list., defaults to None
        :type types: List[ContractTypeEnum], optional
        :param statuses: You can filter contracts by current status, a contract is included in results if its status is in this list., defaults to None
        :type statuses: List[ContractStatusEnum], optional
        :param team_id: Filter contracts for given team ID; NOTE: technically ALL query parameters are strings or array of strings, defaults to None
        :type team_id: str, optional
        :param external_id: Filter contracts for given external ID, defaults to None
        :type external_id: str, optional
        :param countries: Country codes of contracts to filter., defaults to None
        :type countries: List[str], optional
        :param currencies: Currency codes of contracts to filter., defaults to None
        :type currencies: GetContractListCurrencies, optional
        :param search: Include a contract if by name or contractor name contains given search term., defaults to None
        :type search: str, optional
        :param sort_by: Sort contracts by given field name., defaults to None
        :type sort_by: ContractsSortByEnum, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: ContractListContainer
        """

        Validator(str).is_optional().validate(after_cursor)
        Validator(float).is_optional().min(1).max(99).validate(limit)
        Validator(SortDirEnum).is_optional().validate(order_direction)
        Validator(ContractTypeEnum).is_array().is_optional().validate(types)
        Validator(ContractStatusEnum).is_array().is_optional().validate(statuses)
        Validator(str).is_optional().validate(team_id)
        Validator(str).is_optional().validate(external_id)
        Validator(str).is_array().is_optional().validate(countries)
        Validator(GetContractListCurrencies).is_optional().validate(currencies)
        Validator(str).is_optional().validate(search)
        Validator(ContractsSortByEnum).is_optional().validate(sort_by)

        serialized_request = (
            Serializer(f"{self.base_url}/contracts", self.get_default_headers())
            .add_query("after_cursor", after_cursor)
            .add_query("limit", limit)
            .add_query("order_direction", order_direction)
            .add_query("types", types)
            .add_query("statuses", statuses)
            .add_query("team_id", team_id)
            .add_query("external_id", external_id)
            .add_query("countries", countries)
            .add_query("currencies", currencies)
            .add_query("search", search)
            .add_query("sort_by", sort_by)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return ContractListContainer._unmap(response)

    @cast_models
    def get_contract_by_id(self, contract_id: str) -> ContractContainer:
        """Retrieve a single contract.

        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: ContractContainer
        """

        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}", self.get_default_headers()
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return ContractContainer._unmap(response)

    @cast_models
    def attach_external_id(
        self, request_body: InputToPatchContractExternalId, contract_id: str
    ) -> GenericResultCreated:
        """Add an external Id to a Deel contract. You can use this to add a Deel contract's refernece Id in your platform. External Id can be passed as a query parameter in List contract endpoint to find this conract later.

        :param request_body: The request body.
        :type request_body: InputToPatchContractExternalId
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultCreated
        """

        Validator(InputToPatchContractExternalId).validate(request_body)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}", self.get_default_headers()
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("PATCH")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultCreated._unmap(response)

    @cast_models
    def add_contract_document(
        self, contract_id: str, request_body: dict = None
    ) -> ContractDocumentContainer:
        """Attach a file to contract document.

        :param request_body: The request body., defaults to None
        :type request_body: dict, optional
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: ContractDocumentContainer
        """

        Validator(dict).is_optional().validate(request_body)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/documents",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body, "multipart/form-data")
        )

        response = self.send_request(serialized_request)

        return ContractDocumentContainer._unmap(response)

    @cast_models
    def edit_contract_document(
        self, contract_id: str, request_body: dict = None
    ) -> ContractDocumentContainer:
        """Overwrite the file currently attached to contract document.

        :param request_body: The request body., defaults to None
        :type request_body: dict, optional
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: ContractDocumentContainer
        """

        Validator(dict).is_optional().validate(request_body)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/documents",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("PUT")
            .set_body(request_body, "multipart/form-data")
        )

        response = self.send_request(serialized_request)

        return ContractDocumentContainer._unmap(response)

    @cast_models
    def get_alternate_emails_by_contract_id(
        self, contract_id: str
    ) -> List[AlternateEmailItem]:
        """Returns an array of alternate email objects

        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: List[AlternateEmailItem]
        """

        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/alternate_emails",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return [AlternateEmailItem._unmap(item) for item in response]

    @cast_models
    def sign_contract(
        self, request_body: ContractSignatureToCreateContainer, contract_id: str
    ) -> GenericResultCreated:
        """Sign a contract as a client.

        :param request_body: The request body.
        :type request_body: ContractSignatureToCreateContainer
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultCreated
        """

        Validator(ContractSignatureToCreateContainer).validate(request_body)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/signatures",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultCreated._unmap(response)

    @cast_models
    def archive_contract(self, contract_id: str) -> GenericResultUpdated:
        """Archive a terminated, cancelled or completed contract.

        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultUpdated
        """

        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/archive",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("PATCH")
        )

        response = self.send_request(serialized_request)

        return GenericResultUpdated._unmap(response)

    @cast_models
    def invite_to_sign_contract(
        self, request_body: ContractInvitationToCreateContainer, contract_id: str
    ) -> GenericResultCreated:
        """Invite a worker to sign the contract. Worker will be notified via email.

        :param request_body: The request body.
        :type request_body: ContractInvitationToCreateContainer
        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultCreated
        """

        Validator(ContractInvitationToCreateContainer).validate(request_body)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/invitations",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultCreated._unmap(response)

    @cast_models
    def uninvite_to_sign_contract(self, contract_id: str) -> GenericResultDeleted:
        """Remove invite in order to re-invite a worker to sign the contract.

        :param contract_id: Deel contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultDeleted
        """

        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/invitations",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)

        return GenericResultDeleted._unmap(response)

    @cast_models
    def calculate_final_payment(
        self,
        contract_id: str,
        end_date: str = None,
        calculation_type: CalculateFinalPaymentCalculationType = None,
        workweek_start: str = None,
        workweek_end: str = None,
    ) -> FinalPaymentCalculatedContainer:
        """Calculate the final payment due to the contractor when ending the contract.

        :param contract_id: Deel contract id.
        :type contract_id: str
        :param end_date: The day to end the contract., defaults to None
        :type end_date: str, optional
        :param calculation_type: The days to calculate., defaults to None
        :type calculation_type: CalculateFinalPaymentCalculationType, optional
        :param workweek_start: The day the work week starts, 0 to 6, defaults to None
        :type workweek_start: str, optional
        :param workweek_end: The day the work week ends, 0 to 6, defaults to None
        :type workweek_end: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: FinalPaymentCalculatedContainer
        """

        Validator(str).validate(contract_id)
        Validator(str).is_optional().pattern("^\d{4}-\d{2}-\d{2}$").validate(end_date)
        Validator(CalculateFinalPaymentCalculationType).is_optional().validate(
            calculation_type
        )
        Validator(str).is_optional().validate(workweek_start)
        Validator(str).is_optional().validate(workweek_end)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/final-payments",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .add_query("end_date", end_date, nullable=True)
            .add_query("calculation_type", calculation_type)
            .add_query("workweek_start", workweek_start)
            .add_query("workweek_end", workweek_end)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return FinalPaymentCalculatedContainer._unmap(response)

    @cast_models
    def post_contract_estimate(
        self, request_body: EstimateFirstPaymentContainer
    ) -> ResponseEstimateFirstPaymentContainer:
        """First payment is calculated from the number of working/calendar days between their start date and the start of the payment cycle.

        :param request_body: The request body.
        :type request_body: EstimateFirstPaymentContainer
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: ResponseEstimateFirstPaymentContainer
        """

        Validator(EstimateFirstPaymentContainer).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/estimate", self.get_default_headers()
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return ResponseEstimateFirstPaymentContainer._unmap(response)

    @cast_models
    def get_contract_templates(self) -> ContractTemplateListContainer:
        """Retrieve a list of contract templates in your organization.

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: ContractTemplateListContainer
        """

        serialized_request = (
            Serializer(
                f"{self.base_url}/contract-templates", self.get_default_headers()
            )
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return ContractTemplateListContainer._unmap(response)

    @cast_models
    def get_worker_documents_by_id(
        self, worker_id: str
    ) -> WorkerDocumentsByIdContainer:
        """Retrieve a list of documents of a worker.

        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: successful operation
        :rtype: WorkerDocumentsByIdContainer
        """

        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/workers/{{worker_id}}/documents",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return WorkerDocumentsByIdContainer._unmap(response)

    @cast_models
    def get_download_worker_documents_by_id(
        self, worker_id: str, document_id: float
    ) -> DownloadWorkerDocumentsByIdContainer:
        """Get the download link of worker document.

        :param worker_id: Unique identifier for a worker.
        :type worker_id: str
        :param document_id: Unique identifier for a compliance document in Deel.
        :type document_id: float
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: successful operation
        :rtype: DownloadWorkerDocumentsByIdContainer
        """

        Validator(str).validate(worker_id)
        Validator(float).validate(document_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/workers/{{worker_id}}/documents/{{document_id}}/download",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .add_path("document_id", document_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return DownloadWorkerDocumentsByIdContainer._unmap(response)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class FileAttachmentInfo(BaseModel):
    """This object is used for linking file attachments to your records.

    :param filename: Original filename you used to upload using attachments end-point.
    :type filename: str
    :param key: You can call attachments end-point, get key and URL to upload your file.
    :type key: str
    """

    def __init__(self, filename: str, key: str):
        self.filename = filename
        self.key = key



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .identifier_value import IdentifierValue, IdentifierValueGuard
from .contribution import Contribution


@JsonMap({"id_": "id"})
class PensionProvider(BaseModel):
    """Pension provider.

    :param id_: id_, defaults to None
    :type id_: IdentifierValue, optional
    :param name: Name of pension provider., defaults to None
    :type name: str, optional
    :param home_page_url: Pension provider's home page url., defaults to None
    :type home_page_url: str, optional
    :param contribution: Pension contribution., defaults to None
    :type contribution: Contribution, optional
    """

    def __init__(
        self,
        id_: IdentifierValue = None,
        name: str = None,
        home_page_url: str = None,
        contribution: Contribution = None,
    ):
        if id_ is not None:
            self.id_ = IdentifierValueGuard.return_one_of(id_)
        if name is not None:
            self.name = name
        if home_page_url is not None:
            self.home_page_url = home_page_url
        if contribution is not None:
            self.contribution = self._define_object(contribution, Contribution)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .contract_details_to_amend import ContractDetailsToAmend


@JsonMap({})
class ContractToAmendDetailsContainer(BaseModel):
    """ContractToAmendDetailsContainer

    :param data: You can specify any combination of data points that need changing.
    :type data: ContractDetailsToAmend
    """

    def __init__(self, data: ContractDetailsToAmend):
        self.data = self._define_object(data, ContractDetailsToAmend)



# This file was generated by liblab | https://liblab.com/

from typing import Union
from typing import List
from .base import OneOfBaseModel


class GetContractListCurrenciesGuard(OneOfBaseModel):
    class_list = {"List[str]": List[str], "str": str}


GetContractListCurrencies = Union[List[str], str]



# This file was generated by liblab | https://liblab.com/

from .services.accounting import AccountingService
from .services.managers import ManagersService
from .services.people import PeopleService
from .services.eor import EorService
from .services.global_payroll import GlobalPayrollService
from .services.contractors import ContractorsService
from .services.adjustments import AdjustmentsService
from .services.candidates import CandidatesService
from .services.partner_managed import PartnerManagedService
from .services.contracts import ContractsService
from .services.tasks import TasksService
from .services.timesheets import TimesheetsService
from .services.milestones import MilestonesService
from .services.off_cycle_payments import OffCyclePaymentsService
from .services.time_off import TimeOffService
from .services.invoices import InvoicesService
from .services.organizations import OrganizationsService
from .services.lookups import LookupsService
from .services.webhooks import WebhooksService
from .services.token import TokenService
from .services.carta import CartaService
from .net.environment import Environment


class DeelSdk:
    def __init__(self, access_token: str, base_url: str = Environment.DEFAULT.value):
        """
        Initializes DeelSdk the SDK class.
        """
        self.accounting = AccountingService(base_url=base_url)
        self.managers = ManagersService(base_url=base_url)
        self.people = PeopleService(base_url=base_url)
        self.eor = EorService(base_url=base_url)
        self.global_payroll = GlobalPayrollService(base_url=base_url)
        self.contractors = ContractorsService(base_url=base_url)
        self.adjustments = AdjustmentsService(base_url=base_url)
        self.candidates = CandidatesService(base_url=base_url)
        self.partner_managed = PartnerManagedService(base_url=base_url)
        self.contracts = ContractsService(base_url=base_url)
        self.tasks = TasksService(base_url=base_url)
        self.timesheets = TimesheetsService(base_url=base_url)
        self.milestones = MilestonesService(base_url=base_url)
        self.off_cycle_payments = OffCyclePaymentsService(base_url=base_url)
        self.time_off = TimeOffService(base_url=base_url)
        self.invoices = InvoicesService(base_url=base_url)
        self.organizations = OrganizationsService(base_url=base_url)
        self.lookups = LookupsService(base_url=base_url)
        self.webhooks = WebhooksService(base_url=base_url)
        self.token = TokenService(base_url=base_url)
        self.carta = CartaService(base_url=base_url)
        self.set_access_token(access_token)

    def set_base_url(self, base_url):
        """
        Sets the base URL for the entire SDK.
        """
        self.accounting.set_base_url(base_url)
        self.managers.set_base_url(base_url)
        self.people.set_base_url(base_url)
        self.eor.set_base_url(base_url)
        self.global_payroll.set_base_url(base_url)
        self.contractors.set_base_url(base_url)
        self.adjustments.set_base_url(base_url)
        self.candidates.set_base_url(base_url)
        self.partner_managed.set_base_url(base_url)
        self.contracts.set_base_url(base_url)
        self.tasks.set_base_url(base_url)
        self.timesheets.set_base_url(base_url)
        self.milestones.set_base_url(base_url)
        self.off_cycle_payments.set_base_url(base_url)
        self.time_off.set_base_url(base_url)
        self.invoices.set_base_url(base_url)
        self.organizations.set_base_url(base_url)
        self.lookups.set_base_url(base_url)
        self.webhooks.set_base_url(base_url)
        self.token.set_base_url(base_url)
        self.carta.set_base_url(base_url)

        return self

    def set_access_token(self, access_token: str):
        """
        Sets the access token for the entire SDK.
        """
        self.accounting.set_access_token(access_token)
        self.managers.set_access_token(access_token)
        self.people.set_access_token(access_token)
        self.eor.set_access_token(access_token)
        self.global_payroll.set_access_token(access_token)
        self.contractors.set_access_token(access_token)
        self.adjustments.set_access_token(access_token)
        self.candidates.set_access_token(access_token)
        self.partner_managed.set_access_token(access_token)
        self.contracts.set_access_token(access_token)
        self.tasks.set_access_token(access_token)
        self.timesheets.set_access_token(access_token)
        self.milestones.set_access_token(access_token)
        self.off_cycle_payments.set_access_token(access_token)
        self.time_off.set_access_token(access_token)
        self.invoices.set_access_token(access_token)
        self.organizations.set_access_token(access_token)
        self.lookups.set_access_token(access_token)
        self.webhooks.set_access_token(access_token)
        self.token.set_access_token(access_token)
        self.carta.set_access_token(access_token)

        return self


# c029837e0e474b76bc487506e8799df5e3335891efe4fb02bda7a1441840310c



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .invoice_adjustment_review_to_create import InvoiceAdjustmentReviewToCreate


@JsonMap({})
class InvoiceAdjustmentReviewToCreateContainer(BaseModel):
    """InvoiceAdjustmentReviewToCreateContainer

    :param data: data
    :type data: InvoiceAdjustmentReviewToCreate
    """

    def __init__(self, data: InvoiceAdjustmentReviewToCreate):
        self.data = self._define_object(data, InvoiceAdjustmentReviewToCreate)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .contract_type_enum import ContractTypeEnum
from .contract_status_enum import ContractStatusEnum
from .client_of_basic_contract import ClientOfBasicContract
from .worker_of_basic_contract import WorkerOfBasicContract
from .invitations_of_basic_contract import InvitationsOfBasicContract
from .signatures_of_basic_contract import SignaturesOfBasicContract
from .contract_who_reports_enum import ContractWhoReportsEnum


@JsonMap({"id_": "id", "type_": "type"})
class BasicContract(BaseModel):
    """BasicContract

    :param id_: id_
    :type id_: str
    :param title: title
    :type title: str
    :param type_: Type of a contract
    :type type_: ContractTypeEnum
    :param status: Status of a contract in Deel workflow
    :type status: ContractStatusEnum
    :param client: client
    :type client: ClientOfBasicContract
    :param worker: worker
    :type worker: WorkerOfBasicContract
    :param invitations: invitations
    :type invitations: InvitationsOfBasicContract
    :param signatures: signatures
    :type signatures: SignaturesOfBasicContract
    :param is_shielded: is_shielded
    :type is_shielded: bool
    :param is_archived: is_archived
    :type is_archived: bool
    :param created_at: Long date-time format following ISO-8601
    :type created_at: str
    :param termination_date: Long date-time format following ISO-8601
    :type termination_date: str
    :param notice_period: Days before to notice the termination of contract for eather party., defaults to None
    :type notice_period: float, optional
    :param scale: The payment scale (e.g., hourly, weekly, monthly, etc.), defaults to None
    :type scale: str, optional
    :param who_reports: Flag to indicate who is supposed to provide regular reports, defaults to None
    :type who_reports: ContractWhoReportsEnum, optional
    :param external_id: A unique identifier for the object provided by an external system., defaults to None
    :type external_id: str, optional
    """

    def __init__(
        self,
        id_: str,
        title: str,
        type_: ContractTypeEnum,
        status: ContractStatusEnum,
        client: ClientOfBasicContract,
        worker: WorkerOfBasicContract,
        invitations: InvitationsOfBasicContract,
        signatures: SignaturesOfBasicContract,
        is_shielded: bool,
        is_archived: bool,
        created_at: str,
        termination_date: str,
        notice_period: float = None,
        scale: str = None,
        who_reports: ContractWhoReportsEnum = None,
        external_id: str = None,
    ):
        self.id_ = id_
        self.title = title
        self.type_ = self._enum_matching(type_, ContractTypeEnum.list(), "type_")
        self.status = self._enum_matching(status, ContractStatusEnum.list(), "status")
        self.client = self._define_object(client, ClientOfBasicContract)
        self.worker = self._define_object(worker, WorkerOfBasicContract)
        self.invitations = self._define_object(invitations, InvitationsOfBasicContract)
        self.signatures = self._define_object(signatures, SignaturesOfBasicContract)
        self.is_shielded = is_shielded
        self.is_archived = is_archived
        self.created_at = created_at
        self.termination_date = termination_date
        if notice_period is not None:
            self.notice_period = notice_period
        if scale is not None:
            self.scale = scale
        if who_reports is not None:
            self.who_reports = self._enum_matching(
                who_reports, ContractWhoReportsEnum.list(), "who_reports"
            )
        if external_id is not None:
            self.external_id = external_id



# This file was generated by liblab | https://liblab.com/

from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class Costs(BaseModel):
    """Costs

    :param name: The name of the cost item., defaults to None
    :type name: str, optional
    :param amount: The amount or price of the cost item., defaults to None
    :type amount: str, optional
    :param frequency: The frequency that the cost item must be paid., defaults to None
    :type frequency: str, optional
    :param country: The country that the cost item is associated with., defaults to None
    :type country: str, optional
    :param country_code: The country code for the country that the cost item is associated with., defaults to None
    :type country_code: str, optional
    """

    def __init__(
        self,
        name: str = None,
        amount: str = None,
        frequency: str = None,
        country: str = None,
        country_code: str = None,
    ):
        if name is not None:
            self.name = name
        if amount is not None:
            self.amount = amount
        if frequency is not None:
            self.frequency = frequency
        if country is not None:
            self.country = country
        if country_code is not None:
            self.country_code = country_code


@JsonMap({})
class AdditionalData(BaseModel):
    """Textual notes describing important additional data regarding the employement process or costs.

    :param additional_notes: additional_notes, defaults to None
    :type additional_notes: List[str], optional
    """

    def __init__(self, additional_notes: List[str] = None):
        if additional_notes is not None:
            self.additional_notes = additional_notes


@JsonMap({})
class BenefitsData(BaseModel):
    """BenefitsData

    :param name: The name of the benefit item., defaults to None
    :type name: str, optional
    :param plan: The name of the benefit plan., defaults to None
    :type plan: str, optional
    :param fee_amount: The amount or price of the fee associated with the benefit item., defaults to None
    :type fee_amount: str, optional
    :param amount: The amount or price of the benefit item., defaults to None
    :type amount: str, optional
    """

    def __init__(
        self,
        name: str = None,
        plan: str = None,
        fee_amount: str = None,
        amount: str = None,
    ):
        if name is not None:
            self.name = name
        if plan is not None:
            self.plan = plan
        if fee_amount is not None:
            self.fee_amount = fee_amount
        if amount is not None:
            self.amount = amount


@JsonMap({})
class EorEmployeeCostCalculationResponseContainer(BaseModel):
    """EorEmployeeCostCalculationResponseContainer

    :param salary: The gross annual salary of the employee., defaults to None
    :type salary: str, optional
    :param currency: The currency of the employee salary., defaults to None
    :type currency: str, optional
    :param country: The country of employment., defaults to None
    :type country: str, optional
    :param country_code: The country code for the country of employment., defaults to None
    :type country_code: str, optional
    :param frequency: The frequency that the salary is paid out to the employee., defaults to None
    :type frequency: str, optional
    :param deel_fee: The monthly Deel fee., defaults to None
    :type deel_fee: str, optional
    :param severance_accural: Monthly funds to cover future potential severance payments to the employee., defaults to None
    :type severance_accural: str, optional
    :param total_costs: Total monthly cost including salary, Deel fee, severance accrual and employer costs., defaults to None
    :type total_costs: str, optional
    :param employer_costs: Total monthly employer costs. Does not include Deel fee, salary and severance accrual., defaults to None
    :type employer_costs: str, optional
    :param costs: The various costs associated with employment in the specified country., defaults to None
    :type costs: List[Costs], optional
    :param additional_data: Textual notes describing important additional data regarding the employement process or costs., defaults to None
    :type additional_data: AdditionalData, optional
    :param benefits_data: The various benefits costs involved in the calculation, if any., defaults to None
    :type benefits_data: List[BenefitsData], optional
    """

    def __init__(
        self,
        salary: str = None,
        currency: str = None,
        country: str = None,
        country_code: str = None,
        frequency: str = None,
        deel_fee: str = None,
        severance_accural: str = None,
        total_costs: str = None,
        employer_costs: str = None,
        costs: List[Costs] = None,
        additional_data: AdditionalData = None,
        benefits_data: List[BenefitsData] = None,
    ):
        if salary is not None:
            self.salary = salary
        if currency is not None:
            self.currency = currency
        if country is not None:
            self.country = country
        if country_code is not None:
            self.country_code = country_code
        if frequency is not None:
            self.frequency = frequency
        if deel_fee is not None:
            self.deel_fee = deel_fee
        if severance_accural is not None:
            self.severance_accural = severance_accural
        if total_costs is not None:
            self.total_costs = total_costs
        if employer_costs is not None:
            self.employer_costs = employer_costs
        if costs is not None:
            self.costs = self._define_list(costs, Costs)
        if additional_data is not None:
            self.additional_data = self._define_object(additional_data, AdditionalData)
        if benefits_data is not None:
            self.benefits_data = self._define_list(benefits_data, BenefitsData)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .identifier_value import IdentifierValue, IdentifierValueGuard


@JsonMap({"id_": "id"})
class ContractTemplateSummary(BaseModel):
    """ContractTemplateSummary

    :param id_: id_
    :type id_: IdentifierValue
    :param title: Title of a contract template
    :type title: str
    """

    def __init__(self, id_: IdentifierValue, title: str):
        self.id_ = IdentifierValueGuard.return_one_of(id_)
        self.title = title



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class GpEmployeeAddressUpdateData(BaseModel):
    """GpEmployeeAddressUpdateData

    :param city: City name., defaults to None
    :type city: str, optional
    :param street: Street name., defaults to None
    :type street: str, optional
    :param zip: Zip code., defaults to None
    :type zip: str, optional
    """

    def __init__(self, city: str = None, street: str = None, zip: str = None):
        if city is not None:
            self.city = city
        if street is not None:
            self.street = street
        if zip is not None:
            self.zip = zip



# This file was generated by liblab | https://liblab.com/

from enum import Enum
from .utils.json_map import JsonMap
from .base import BaseModel


class AgreementReflectsRelation(Enum):
    """An enumeration representing different categories.

    :cvar TRUE: "true"
    :vartype TRUE: str
    :cvar FALSE: "false"
    :vartype FALSE: str
    """

    TRUE = "true"
    FALSE = "false"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, AgreementReflectsRelation._member_map_.values())
        )


class ContractorCharacteristics(Enum):
    """An enumeration representing different categories.

    :cvar TRUE: "true"
    :vartype TRUE: str
    :cvar FALSE: "false"
    :vartype FALSE: str
    """

    TRUE = "true"
    FALSE = "false"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, ContractorCharacteristics._member_map_.values())
        )


@JsonMap({})
class PremiumToAdd(BaseModel):
    """PremiumToAdd

    :param agreement_reflects_relation: Is the Contractor Agreement selected on the Deel Platform perfectly reflecting your organizations relation and actual working practices with the contractor?
    :type agreement_reflects_relation: AgreementReflectsRelation
    :param contractor_characteristics: Workers doing certain jobs may be considered to be employees by law, even if they would otherwise be considered independent contractors under common law. Is the work of this worker characterized as: Apprentice, Trainee, Labourer, Driver, Medical worker, Legal worker, Construction worker or someone working in the fields of Finance/Investment?
    :type contractor_characteristics: ContractorCharacteristics
    """

    def __init__(
        self,
        agreement_reflects_relation: AgreementReflectsRelation,
        contractor_characteristics: ContractorCharacteristics,
    ):
        self.agreement_reflects_relation = self._enum_matching(
            agreement_reflects_relation,
            AgreementReflectsRelation.list(),
            "agreement_reflects_relation",
        )
        self.contractor_characteristics = self._enum_matching(
            contractor_characteristics,
            ContractorCharacteristics.list(),
            "contractor_characteristics",
        )



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class ContractsSortByEnum(Enum):
    """An enumeration representing different categories.

    :cvar CONTRACT_TITLE: "contract_title"
    :vartype CONTRACT_TITLE: str
    :cvar WORKER_NAME: "worker_name"
    :vartype WORKER_NAME: str
    """

    CONTRACT_TITLE = "contract_title"
    WORKER_NAME = "worker_name"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ContractsSortByEnum._member_map_.values()))



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .base import BaseModel
from .pension_eor_contract_to_create import PensionEorContractToCreate


@JsonMap({})
class EmployeeAddress1(BaseModel):
    """EmployeeAddress1

    :param street: Street and number., defaults to None
    :type street: str, optional
    :param city: City name., defaults to None
    :type city: str, optional
    :param state: State code., defaults to None
    :type state: str, optional
    :param zip: Zip code., defaults to None
    :type zip: str, optional
    :param country: Country code., defaults to None
    :type country: str, optional
    """

    def __init__(
        self,
        street: str = None,
        city: str = None,
        state: str = None,
        zip: str = None,
        country: str = None,
    ):
        if street is not None:
            self.street = street
        if city is not None:
            self.city = city
        if state is not None:
            self.state = state
        if zip is not None:
            self.zip = zip
        if country is not None:
            self.country = self._pattern_matching(country, "^[A-Z]{2}$", "country")


@JsonMap({})
class EorContractToCreateEmployee(BaseModel):
    """EorContractToCreateEmployee

    :param first_name: Employee's first name.
    :type first_name: str
    :param last_name: Employee's last name.
    :type last_name: str
    :param email: Employee's email address., defaults to None
    :type email: str, optional
    :param nationality: Employee's nationality.
    :type nationality: str
    :param address: address, defaults to None
    :type address: EmployeeAddress1, optional
    """

    def __init__(
        self,
        first_name: str,
        last_name: str,
        nationality: str,
        email: str = None,
        address: EmployeeAddress1 = None,
    ):
        self.first_name = first_name
        self.last_name = last_name
        if email is not None:
            self.email = email
        self.nationality = nationality
        if address is not None:
            self.address = self._define_object(address, EmployeeAddress1)


class EmploymentType(Enum):
    """An enumeration representing different categories.

    :cvar FULL_TIME: "Full-time"
    :vartype FULL_TIME: str
    :cvar PART_TIME: "Part-time"
    :vartype PART_TIME: str
    """

    FULL_TIME = "Full-time"
    PART_TIME = "Part-time"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, EmploymentType._member_map_.values()))


class TimeOffType(Enum):
    """An enumeration representing different categories.

    :cvar STANDARD: "STANDARD"
    :vartype STANDARD: str
    :cvar SPECIFIC: "SPECIFIC"
    :vartype SPECIFIC: str
    """

    STANDARD = "STANDARD"
    SPECIFIC = "SPECIFIC"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, TimeOffType._member_map_.values()))


@JsonMap({"type_": "type"})
class EorContractToCreateEmployment(BaseModel):
    """EorContractToCreateEmployment

    :param country: Country code.
    :type country: str
    :param state: State code of the state/province where the this person will be employed., defaults to None
    :type state: str, optional
    :param type_: Is it a full-time contract or a part-time contract?, defaults to None
    :type type_: EmploymentType, optional
    :param work_visa_required: Do you require Deel to apply for work visa for this person?
    :type work_visa_required: bool
    :param start_date: Short date in format ISO-8601 (YYYY-MM-DD). For example: 2022-12-31.
    :type start_date: str
    :param end_date: Short date in format ISO-8601 (YYYY-MM-DD). For example 2022-12-31., defaults to None
    :type end_date: str, optional
    :param probation_period: Number of probation days., defaults to None
    :type probation_period: float, optional
    :param scope_of_work: Scope of work description., defaults to None
    :type scope_of_work: str, optional
    :param time_off_type: If you want to use standard number of holidays for this employee, choose "STANDARD". If you want to enter a specific number of holidays, choose "SPECIFIC" and enter the number of days in the holidays field., defaults to None
    :type time_off_type: TimeOffType, optional
    :param holidays: Enter the number of holidays. Leave this field blank if you are chooseing "STANDARD" time_off_type., defaults to None
    :type holidays: float, optional
    """

    def __init__(
        self,
        country: str,
        work_visa_required: bool,
        start_date: str,
        state: str = None,
        type_: EmploymentType = None,
        end_date: str = None,
        probation_period: float = None,
        scope_of_work: str = None,
        time_off_type: TimeOffType = None,
        holidays: float = None,
    ):
        self.country = self._pattern_matching(country, "^[A-Z]{2}$", "country")
        if state is not None:
            self.state = state
        if type_ is not None:
            self.type_ = self._enum_matching(type_, EmploymentType.list(), "type_")
        self.work_visa_required = work_visa_required
        self.start_date = start_date
        if end_date is not None:
            self.end_date = self._pattern_matching(
                end_date, "^\d{4}-\d{2}-\d{2}$", "end_date"
            )
        if probation_period is not None:
            self.probation_period = probation_period
        if scope_of_work is not None:
            self.scope_of_work = scope_of_work
        if time_off_type is not None:
            self.time_off_type = self._enum_matching(
                time_off_type, TimeOffType.list(), "time_off_type"
            )
        if holidays is not None:
            self.holidays = holidays


@JsonMap({"id_": "id"})
class EorContractToCreateSeniority(BaseModel):
    """EorContractToCreateSeniority

    :param id_: Unique identifier of this resource., defaults to None
    :type id_: str, optional
    """

    def __init__(self, id_: str = None):
        if id_ is not None:
            self.id_ = id_


@JsonMap({"id_": "id"})
class ClientLegalEntity1(BaseModel):
    """ClientLegalEntity1

    :param id_: Unique identifier of this resource., defaults to None
    :type id_: str, optional
    """

    def __init__(self, id_: str = None):
        if id_ is not None:
            self.id_ = id_


@JsonMap({"id_": "id"})
class ClientTeam1(BaseModel):
    """ClientTeam1

    :param id_: Unique identifier of this resource.
    :type id_: str
    """

    def __init__(self, id_: str):
        self.id_ = id_


@JsonMap({})
class EorContractToCreateClient(BaseModel):
    """EorContractToCreateClient

    :param legal_entity: legal_entity, defaults to None
    :type legal_entity: ClientLegalEntity1, optional
    :param team: team
    :type team: ClientTeam1
    """

    def __init__(self, team: ClientTeam1, legal_entity: ClientLegalEntity1 = None):
        if legal_entity is not None:
            self.legal_entity = self._define_object(legal_entity, ClientLegalEntity1)
        self.team = self._define_object(team, ClientTeam1)


class VariableCompensationType(Enum):
    """An enumeration representing different categories.

    :cvar PERCENTAGE: "PERCENTAGE"
    :vartype PERCENTAGE: str
    :cvar FIXED: "FIXED"
    :vartype FIXED: str
    """

    PERCENTAGE = "PERCENTAGE"
    FIXED = "FIXED"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, VariableCompensationType._member_map_.values())
        )


@JsonMap({})
class EorContractToCreateCompensationDetails(BaseModel):
    """EorContractToCreateCompensationDetails

    :param salary: Employee's gross annual salary., defaults to None
    :type salary: float, optional
    :param currency: Salary currency.
    :type currency: str
    :param variable_compensation: Variable compensation. For example, if the person will earn 5% of the gross annual salary as the variable compensation, enter 5., defaults to None
    :type variable_compensation: float, optional
    :param variable_compensation_type: Should be set only if variable_compensation is set. It can be PERCENTAGE or FIXED., defaults to None
    :type variable_compensation_type: VariableCompensationType, optional
    """

    def __init__(
        self,
        currency: str,
        salary: float = None,
        variable_compensation: float = None,
        variable_compensation_type: VariableCompensationType = None,
    ):
        if salary is not None:
            self.salary = salary
        self.currency = currency
        if variable_compensation is not None:
            self.variable_compensation = variable_compensation
        if variable_compensation_type is not None:
            self.variable_compensation_type = self._enum_matching(
                variable_compensation_type,
                VariableCompensationType.list(),
                "variable_compensation_type",
            )


class WorkerType(Enum):
    """An enumeration representing different categories.

    :cvar SKILLED: "Skilled"
    :vartype SKILLED: str
    :cvar UNSKILLED: "Unskilled"
    :vartype UNSKILLED: str
    """

    SKILLED = "Skilled"
    UNSKILLED = "Unskilled"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, WorkerType._member_map_.values()))


@JsonMap({})
class QuoteAdditionalFields(BaseModel):
    """Some countries require additional employee information for employment contracts.

    :param gender: Employee's gender., defaults to None
    :type gender: str, optional
    :param worker_type: Type of worker., defaults to None
    :type worker_type: WorkerType, optional
    :param dob: Employee's date of birth., defaults to None
    :type dob: str, optional
    """

    def __init__(
        self, gender: str = None, worker_type: WorkerType = None, dob: str = None
    ):
        if gender is not None:
            self.gender = gender
        if worker_type is not None:
            self.worker_type = self._enum_matching(
                worker_type, WorkerType.list(), "worker_type"
            )
        if dob is not None:
            self.dob = dob


@JsonMap({})
class EorContractToCreate(BaseModel):
    """EorContractToCreate

    :param employee: employee
    :type employee: EorContractToCreateEmployee
    :param employment: employment
    :type employment: EorContractToCreateEmployment
    :param job_title: Employee's job title.
    :type job_title: str
    :param seniority: seniority
    :type seniority: EorContractToCreateSeniority
    :param client: client
    :type client: EorContractToCreateClient
    :param compensation_details: compensation_details
    :type compensation_details: EorContractToCreateCompensationDetails
    :param quote_additional_fields: Some countries require additional employee information for employment contracts., defaults to None
    :type quote_additional_fields: QuoteAdditionalFields, optional
    :param health_plan_id: Healthcare plan id. You can see available healthcare plans in the country guide endpoint., defaults to None
    :type health_plan_id: str, optional
    :param pension: Some countries required a pension benefit., defaults to None
    :type pension: PensionEorContractToCreate, optional
    """

    def __init__(
        self,
        employee: EorContractToCreateEmployee,
        employment: EorContractToCreateEmployment,
        job_title: str,
        seniority: EorContractToCreateSeniority,
        client: EorContractToCreateClient,
        compensation_details: EorContractToCreateCompensationDetails,
        quote_additional_fields: QuoteAdditionalFields = None,
        health_plan_id: str = None,
        pension: PensionEorContractToCreate = None,
    ):
        self.employee = self._define_object(employee, EorContractToCreateEmployee)
        self.employment = self._define_object(employment, EorContractToCreateEmployment)
        self.job_title = job_title
        self.seniority = self._define_object(seniority, EorContractToCreateSeniority)
        self.client = self._define_object(client, EorContractToCreateClient)
        self.compensation_details = self._define_object(
            compensation_details, EorContractToCreateCompensationDetails
        )
        if quote_additional_fields is not None:
            self.quote_additional_fields = self._define_object(
                quote_additional_fields, QuoteAdditionalFields
            )
        if health_plan_id is not None:
            self.health_plan_id = health_plan_id
        if pension is not None:
            self.pension = self._define_object(pension, PensionEorContractToCreate)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .benefit_contribution_type_enum import BenefitContributionTypeEnum


@JsonMap({"type_": "type"})
class Contribution(BaseModel):
    """Pension contribution.

    :param type_: type_, defaults to None
    :type type_: BenefitContributionTypeEnum, optional
    :param minimum: Minimum pension contribution., defaults to None
    :type minimum: str, optional
    :param maximum: Maximum pension contribution., defaults to None
    :type maximum: str, optional
    """

    def __init__(
        self,
        type_: BenefitContributionTypeEnum = None,
        minimum: str = None,
        maximum: str = None,
    ):
        if type_ is not None:
            self.type_ = self._enum_matching(
                type_, BenefitContributionTypeEnum.list(), "type_"
            )
        if minimum is not None:
            self.minimum = minimum
        if maximum is not None:
            self.maximum = maximum



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.output_to_create_file_ref_container import OutputToCreateFileRefContainer
from ..models.input_to_create_file_ref import InputToCreateFileRef
from ..models.generic_result_updated import GenericResultUpdated
from ..models.generic_result_deleted import GenericResultDeleted
from ..models.adjustments_container import AdjustmentsContainer
from ..models.adjustments_categories_container import AdjustmentsCategoriesContainer
from ..models.adjustment_to_update_container import AdjustmentToUpdateContainer
from ..models.adjustment_to_create_container import AdjustmentToCreateContainer
from ..models.adjustment_created_container import AdjustmentCreatedContainer


class AdjustmentsService(BaseService):

    @cast_models
    def create_adjustment(self, request_body: dict) -> AdjustmentCreatedContainer:
        """Create a new adjustment.

        :param request_body: The request body.
        :type request_body: dict
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: AdjustmentCreatedContainer
        """

        Validator(dict).validate(request_body)

        serialized_request = (
            Serializer(f"{self.base_url}/adjustments", self.get_default_headers())
            .serialize()
            .set_method("POST")
            .set_body(request_body, "multipart/form-data")
        )

        response = self.send_request(serialized_request)

        return AdjustmentCreatedContainer._unmap(response)

    @cast_models
    def get_adjustments_by_id(self, adjustment_id: str) -> AdjustmentCreatedContainer:
        """Retrieve an adjustment.

        :param adjustment_id: Adjustment id.
        :type adjustment_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: AdjustmentCreatedContainer
        """

        Validator(str).validate(adjustment_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/adjustments/{{adjustment_id}}",
                self.get_default_headers(),
            )
            .add_path("adjustment_id", adjustment_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return AdjustmentCreatedContainer._unmap(response)

    @cast_models
    def update_adjustment(
        self, request_body: AdjustmentToUpdateContainer, adjustment_id: str
    ) -> GenericResultUpdated:
        """Update an adjustment.

        :param request_body: The request body.
        :type request_body: AdjustmentToUpdateContainer
        :param adjustment_id: Adjustment id.
        :type adjustment_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultUpdated
        """

        Validator(AdjustmentToUpdateContainer).validate(request_body)
        Validator(str).validate(adjustment_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/adjustments/{{adjustment_id}}",
                self.get_default_headers(),
            )
            .add_path("adjustment_id", adjustment_id)
            .serialize()
            .set_method("PATCH")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GenericResultUpdated._unmap(response)

    @cast_models
    def delete_adjustment(self, adjustment_id: str) -> GenericResultDeleted:
        """Delete an adjustment.

        :param adjustment_id: Adjustment id.
        :type adjustment_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GenericResultDeleted
        """

        Validator(str).validate(adjustment_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/adjustments/{{adjustment_id}}",
                self.get_default_headers(),
            )
            .add_path("adjustment_id", adjustment_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)

        return GenericResultDeleted._unmap(response)

    @cast_models
    def get_categories(self) -> AdjustmentsCategoriesContainer:
        """Get all categories for your organization.

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: AdjustmentsCategoriesContainer
        """

        serialized_request = (
            Serializer(
                f"{self.base_url}/adjustments/categories", self.get_default_headers()
            )
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return AdjustmentsCategoriesContainer._unmap(response)

    @cast_models
    def create_file_ref(
        self, request_body: InputToCreateFileRef
    ) -> OutputToCreateFileRefContainer:
        """Upload file to Deel storage to use the file attachment feature for other endpoints.

        :param request_body: The request body.
        :type request_body: InputToCreateFileRef
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: OutputToCreateFileRefContainer
        """

        Validator(InputToCreateFileRef).validate(request_body)

        serialized_request = (
            Serializer(f"{self.base_url}/attachments", self.get_default_headers())
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return OutputToCreateFileRefContainer._unmap(response)

    @cast_models
    def get_adjustments(
        self, contract_id: str, from_: str = None, to: str = None
    ) -> AdjustmentsContainer:
        """Get all adjustments for the specific contract.

        :param contract_id: Deel contract id.
        :type contract_id: str
        :param from_: Filter adjustments by start date., defaults to None
        :type from_: str, optional
        :param to: Filter adjustments by end date., defaults to None
        :type to: str, optional
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: AdjustmentsContainer
        """

        Validator(str).validate(contract_id)
        Validator(str).is_optional().validate(from_)
        Validator(str).is_optional().validate(to)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/adjustments",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .add_query("from", from_)
            .add_query("to", to)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return AdjustmentsContainer._unmap(response)



# This file was generated by liblab | https://liblab.com/

from enum import Enum


class InvoiceAdjustmentTypeEnum(Enum):
    """An enumeration representing different categories.

    :cvar ACCRUED_HOLIDAY: "accrued_holiday"
    :vartype ACCRUED_HOLIDAY: str
    :cvar ADDITIONAL_FEE: "additional_fee"
    :vartype ADDITIONAL_FEE: str
    :cvar ALLOWANCE: "allowance"
    :vartype ALLOWANCE: str
    :cvar BONUS: "bonus"
    :vartype BONUS: str
    :cvar COMMISSION: "commission"
    :vartype COMMISSION: str
    :cvar DEDUCTION: "deduction"
    :vartype DEDUCTION: str
    :cvar DEPOSIT: "deposit"
    :vartype DEPOSIT: str
    :cvar DEPOSIT_REFUND: "deposit_refund"
    :vartype DEPOSIT_REFUND: str
    :cvar EMPLOYER_COST: "employer_cost"
    :vartype EMPLOYER_COST: str
    :cvar EXPENSE: "expense"
    :vartype EXPENSE: str
    :cvar HEALTH_ALLOWANCE: "health_allowance"
    :vartype HEALTH_ALLOWANCE: str
    :cvar HEALTH_BENEFIT: "health_benefit"
    :vartype HEALTH_BENEFIT: str
    :cvar HEALTH_INSURANCE_FEE: "health_insurance_fee"
    :vartype HEALTH_INSURANCE_FEE: str
    :cvar LEGAL_FEE: "legal_fee"
    :vartype LEGAL_FEE: str
    :cvar MANAGEMENT_FEE: "management_fee"
    :vartype MANAGEMENT_FEE: str
    :cvar MILESTONE: "milestone"
    :vartype MILESTONE: str
    :cvar OFFCYCLE: "offcycle"
    :vartype OFFCYCLE: str
    :cvar OTHER: "other"
    :vartype OTHER: str
    :cvar OVERTIME: "overtime"
    :vartype OVERTIME: str
    :cvar PENSION: "pension"
    :vartype PENSION: str
    :cvar PRO_RATA: "pro_rata"
    :vartype PRO_RATA: str
    :cvar SETUP_FEE: "setup_fee"
    :vartype SETUP_FEE: str
    :cvar SEVERANCE: "severance"
    :vartype SEVERANCE: str
    :cvar SHIELD_SERVICE: "shield_service"
    :vartype SHIELD_SERVICE: str
    :cvar SIGNING_BONUS: "signing_bonus"
    :vartype SIGNING_BONUS: str
    :cvar SIGNING_BONUS_EMPLOYER_COST: "signing_bonus_employer_cost"
    :vartype SIGNING_BONUS_EMPLOYER_COST: str
    :cvar REFUND: "refund"
    :vartype REFUND: str
    :cvar TASK: "task"
    :vartype TASK: str
    :cvar TIME_OFF: "time_off"
    :vartype TIME_OFF: str
    :cvar VAT: "vat"
    :vartype VAT: str
    :cvar WITHHOLDING_TAX: "withholding_tax"
    :vartype WITHHOLDING_TAX: str
    :cvar WORK: "work"
    :vartype WORK: str
    """

    ACCRUED_HOLIDAY = "accrued_holiday"
    ADDITIONAL_FEE = "additional_fee"
    ALLOWANCE = "allowance"
    BONUS = "bonus"
    COMMISSION = "commission"
    DEDUCTION = "deduction"
    DEPOSIT = "deposit"
    DEPOSIT_REFUND = "deposit_refund"
    EMPLOYER_COST = "employer_cost"
    EXPENSE = "expense"
    HEALTH_ALLOWANCE = "health_allowance"
    HEALTH_BENEFIT = "health_benefit"
    HEALTH_INSURANCE_FEE = "health_insurance_fee"
    LEGAL_FEE = "legal_fee"
    MANAGEMENT_FEE = "management_fee"
    MILESTONE = "milestone"
    OFFCYCLE = "offcycle"
    OTHER = "other"
    OVERTIME = "overtime"
    PENSION = "pension"
    PRO_RATA = "pro_rata"
    SETUP_FEE = "setup_fee"
    SEVERANCE = "severance"
    SHIELD_SERVICE = "shield_service"
    SIGNING_BONUS = "signing_bonus"
    SIGNING_BONUS_EMPLOYER_COST = "signing_bonus_employer_cost"
    REFUND = "refund"
    TASK = "task"
    TIME_OFF = "time_off"
    VAT = "vat"
    WITHHOLDING_TAX = "withholding_tax"
    WORK = "work"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, InvoiceAdjustmentTypeEnum._member_map_.values())
        )



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id"})
class GpPayrollEventReports(BaseModel):
    """GpPayrollEventReports

    :param id_: id_
    :type id_: str
    :param status: status
    :type status: str
    :param start_date: Long date-time format following ISO-8601
    :type start_date: str
    :param lock_date: Long date-time format following ISO-8601
    :type lock_date: str
    :param end_date: Long date-time format following ISO-8601
    :type end_date: str
    """

    def __init__(
        self, id_: str, status: str, start_date: str, lock_date: str, end_date: str
    ):
        self.id_ = id_
        self.status = status
        self.start_date = start_date
        self.lock_date = lock_date
        self.end_date = end_date



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class WorkerTerminationBody(BaseModel):
    """WorkerTerminationBody

    :param desired_end_date: The preferred end date for terminating the worker's engagement.
    :type desired_end_date: str
    :param last_date_of_work: The effective termination date of the worker's employment.
    :type last_date_of_work: str
    :param message: The reason for terminating the worker's engagement.
    :type message: str
    :param is_voluntary: Indicates whether the worker termination is voluntary (true) or involuntary (false).
    :type is_voluntary: bool
    :param severance: severance
    :type severance: dict
    """

    def __init__(
        self,
        desired_end_date: str,
        last_date_of_work: str,
        message: str,
        is_voluntary: bool,
        severance: dict,
    ):
        self.desired_end_date = desired_end_date
        self.last_date_of_work = last_date_of_work
        self.message = message
        self.is_voluntary = is_voluntary
        self.severance = severance



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class EquityStakeholder(BaseModel):
    """EquityStakeholder

    :param stakeholder_id: Unique identifier of the stakeholder., defaults to None
    :type stakeholder_id: str, optional
    :param worker_id: Unique identifier of the worker., defaults to None
    :type worker_id: str, optional
    """

    def __init__(self, stakeholder_id: str = None, worker_id: str = None):
        if stakeholder_id is not None:
            self.stakeholder_id = stakeholder_id
        if worker_id is not None:
            self.worker_id = worker_id



# This file was generated by liblab | https://liblab.com/

from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class EorField(BaseModel):
    """Mandatory EOR field.

    :param label: label, defaults to None
    :type label: str, optional
    :param property_name: property_name, defaults to None
    :type property_name: str, optional
    :param input_type: input_type, defaults to None
    :type input_type: str, optional
    :param is_optional: is_optional, defaults to None
    :type is_optional: bool, optional
    :param options: options, defaults to None
    :type options: List[str], optional
    """

    def __init__(
        self,
        label: str = None,
        property_name: str = None,
        input_type: str = None,
        is_optional: bool = None,
        options: List[str] = None,
    ):
        if label is not None:
            self.label = label
        if property_name is not None:
            self.property_name = property_name
        if input_type is not None:
            self.input_type = input_type
        if is_optional is not None:
            self.is_optional = is_optional
        if options is not None:
            self.options = options



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .candidate_status_enum import CandidateStatusEnum


@JsonMap({"id_": "id"})
class CandidateToCreate(BaseModel):
    """Details of candidate to create

    :param id_: Unique identifier of candidate in your ATS.
    :type id_: str
    :param first_name: Candidate's first name.
    :type first_name: str
    :param last_name: Candidate's last name.
    :type last_name: str
    :param status: Offer status
    :type status: CandidateStatusEnum
    :param start_date: Short date in format ISO-8601 (YYYY-MM-DD). For example: 2022-12-31.
    :type start_date: str
    :param link: Link to candidate's profile in ATS.
    :type link: str
    :param job_title: Job Title., defaults to None
    :type job_title: str, optional
    :param email: Worker's email address., defaults to None
    :type email: str, optional
    :param nationality: nationality, defaults to None
    :type nationality: str, optional
    :param country: Country code., defaults to None
    :type country: str, optional
    :param state: Job location state., defaults to None
    :type state: str, optional
    """

    def __init__(
        self,
        id_: str,
        first_name: str,
        last_name: str,
        status: CandidateStatusEnum,
        start_date: str,
        link: str,
        job_title: str = None,
        email: str = None,
        nationality: str = None,
        country: str = None,
        state: str = None,
    ):
        self.id_ = id_
        self.first_name = first_name
        self.last_name = last_name
        self.status = self._enum_matching(status, CandidateStatusEnum.list(), "status")
        self.start_date = start_date
        self.link = link
        if job_title is not None:
            self.job_title = job_title
        if email is not None:
            self.email = email
        if nationality is not None:
            self.nationality = nationality
        if country is not None:
            self.country = self._pattern_matching(country, "^[A-Z]{2}$", "country")
        if state is not None:
            self.state = state



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from enum import Enum
from .utils.json_map import JsonMap
from .base import BaseModel
from .contract_who_reports_enum import ContractWhoReportsEnum
from .meta_data_of_contract_to_create import MetaDataOfContractToCreate
from .identifier_value import IdentifierValue, IdentifierValueGuard
from .work_statement_cycle_scale_enum import WorkStatementCycleScaleEnum
from .work_statement_cycle_end_type_enum import WorkStatementCycleEndTypeEnum
from .work_statement_payment_due_type_enum import WorkStatementPaymentDueTypeEnum
from .work_statement_scale_enum import WorkStatementScaleEnum


@JsonMap({"id_": "id"})
class ClientLegalEntity4(BaseModel):
    """Choose the Deel legal entity for this contract. Use legal entity endpoint to retrieve a list of legal entities in your organization.

    :param id_: id_
    :type id_: IdentifierValue
    """

    def __init__(self, id_: IdentifierValue):
        self.id_ = IdentifierValueGuard.return_one_of(id_)


@JsonMap({"id_": "id"})
class ClientTeam2(BaseModel):
    """Choose the Deel team for this contract. Use teams endpoint to retrieve a list of teams in your organization.

    :param id_: id_
    :type id_: IdentifierValue
    """

    def __init__(self, id_: IdentifierValue):
        self.id_ = IdentifierValueGuard.return_one_of(id_)


@JsonMap({})
class ContractToCreatePayAsYouGoTimeBasedClient(BaseModel):
    """ContractToCreatePayAsYouGoTimeBasedClient

    :param legal_entity: Choose the Deel legal entity for this contract. Use legal entity endpoint to retrieve a list of legal entities in your organization.
    :type legal_entity: ClientLegalEntity4
    :param team: Choose the Deel team for this contract. Use teams endpoint to retrieve a list of teams in your organization.
    :type team: ClientTeam2
    """

    def __init__(self, legal_entity: ClientLegalEntity4, team: ClientTeam2):
        self.legal_entity = self._define_object(legal_entity, ClientLegalEntity4)
        self.team = self._define_object(team, ClientTeam2)


@JsonMap({"id_": "id"})
class ContractToCreatePayAsYouGoTimeBasedJobTitle(BaseModel):
    """Worker's job title. You can enter a custom job title or use a pre-defiled job title using the Id retrieved in job-titles endpoint.

    :param id_: Unique identifier of this resource., defaults to None
    :type id_: str, optional
    :param name: Job title. Please leave it blank when entering an Id., defaults to None
    :type name: str, optional
    """

    def __init__(self, id_: str = None, name: str = None):
        if id_ is not None:
            self.id_ = id_
        if name is not None:
            self.name = name


@JsonMap({"id_": "id"})
class ContractToCreatePayAsYouGoTimeBasedSeniority(BaseModel):
    """Job seniority level. Please use the seniority levels endpoint to retrieve the list of seniority levels.

    :param id_: id_, defaults to None
    :type id_: IdentifierValue, optional
    """

    def __init__(self, id_: IdentifierValue = None):
        if id_ is not None:
            self.id_ = IdentifierValueGuard.return_one_of(id_)


@JsonMap({})
class ContractToCreatePayAsYouGoTimeBasedWorker(BaseModel):
    """Worker properties

    :param expected_email: Worker's email address.
    :type expected_email: str
    :param first_name: Worker's first name
    :type first_name: str
    :param last_name: Worker's last name, defaults to None
    :type last_name: str, optional
    """

    def __init__(self, expected_email: str, first_name: str, last_name: str = None):
        self.expected_email = expected_email
        self.first_name = first_name
        if last_name is not None:
            self.last_name = last_name


class ContractToCreatePayAsYouGoTimeBasedType(Enum):
    """An enumeration representing different categories.

    :cvar PAY_AS_YOU_GO_TIME_BASED: "pay_as_you_go_time_based"
    :vartype PAY_AS_YOU_GO_TIME_BASED: str
    """

    PAY_AS_YOU_GO_TIME_BASED = "pay_as_you_go_time_based"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                ContractToCreatePayAsYouGoTimeBasedType._member_map_.values(),
            )
        )


@JsonMap({})
class ContractToCreatePayAsYouGoTimeBasedCompensationDetails(BaseModel):
    """ContractToCreatePayAsYouGoTimeBasedCompensationDetails

    :param amount: amount
    :type amount: float
    :param currency_code: currency_code
    :type currency_code: str
    :param frequency: Scale of the invoice cycle.
    :type frequency: WorkStatementCycleScaleEnum
    :param cycle_end: Date invoice cycle ends.
    :type cycle_end: float
    :param cycle_end_type: Cycle end can be weekly 1-7 (Monday = 1) or monthly 1-31.
    :type cycle_end_type: WorkStatementCycleEndTypeEnum
    :param payment_due_type: Type of payment due. If the payment is due on the last day of month, enter 'WITHIN_MONTH'.
    :type payment_due_type: WorkStatementPaymentDueTypeEnum
    :param payment_due_days: payment_due_days
    :type payment_due_days: float
    :param pay_before_weekends: If the payment due is on a weekend, pay on Friday., defaults to None
    :type pay_before_weekends: bool, optional
    :param first_payment_date: Short date in format ISO-8601 (YYYY-MM-DD). For example 2022-12-31., defaults to None
    :type first_payment_date: str, optional
    :param first_payment: First payment amount., defaults to None
    :type first_payment: float, optional
    :param notice_period: Days before to notice the termination of contract for eather party., defaults to None
    :type notice_period: float, optional
    :param scale: Defines the scale at which the amount is paid. For example, enter 'hourly' to define the amount per hour. This field can be excluded when creating a Pay-as-you-go task-based or Milestone contracts.
    :type scale: WorkStatementScaleEnum
    """

    def __init__(
        self,
        amount: float,
        currency_code: str,
        frequency: WorkStatementCycleScaleEnum,
        cycle_end: float,
        cycle_end_type: WorkStatementCycleEndTypeEnum,
        payment_due_type: WorkStatementPaymentDueTypeEnum,
        payment_due_days: float,
        scale: WorkStatementScaleEnum,
        pay_before_weekends: bool = None,
        first_payment_date: str = None,
        first_payment: float = None,
        notice_period: float = None,
    ):
        self.amount = amount
        self.currency_code = self._pattern_matching(
            currency_code, "^[A-Z]{3}$", "currency_code"
        )
        self.frequency = self._enum_matching(
            frequency, WorkStatementCycleScaleEnum.list(), "frequency"
        )
        self.cycle_end = cycle_end
        self.cycle_end_type = self._enum_matching(
            cycle_end_type, WorkStatementCycleEndTypeEnum.list(), "cycle_end_type"
        )
        self.payment_due_type = self._enum_matching(
            payment_due_type, WorkStatementPaymentDueTypeEnum.list(), "payment_due_type"
        )
        self.payment_due_days = payment_due_days
        if pay_before_weekends is not None:
            self.pay_before_weekends = pay_before_weekends
        if first_payment_date is not None:
            self.first_payment_date = self._pattern_matching(
                first_payment_date, "^\d{4}-\d{2}-\d{2}$", "first_payment_date"
            )
        if first_payment is not None:
            self.first_payment = first_payment
        if notice_period is not None:
            self.notice_period = notice_period
        self.scale = self._enum_matching(scale, WorkStatementScaleEnum.list(), "scale")


@JsonMap({"type_": "type"})
class ContractToCreatePayAsYouGoTimeBased(BaseModel):
    """ContractToCreatePayAsYouGoTimeBased

    :param title: The title of the contract.
    :type title: str
    :param country_code: Country code., defaults to None
    :type country_code: str, optional
    :param state_code: The state or province code. Use country lookup endpoint to retrieve state codes., defaults to None
    :type state_code: str, optional
    :param scope_of_work: Describe the work to be performed., defaults to None
    :type scope_of_work: str, optional
    :param special_clause: Enter any special clause you may have., defaults to None
    :type special_clause: str, optional
    :param termination_date: Short date in format ISO-8601 (YYYY-MM-DD). For example 2022-12-31., defaults to None
    :type termination_date: str, optional
    :param client: client
    :type client: ContractToCreatePayAsYouGoTimeBasedClient
    :param job_title: Worker's job title. You can enter a custom job title or use a pre-defiled job title using the Id retrieved in job-titles endpoint.
    :type job_title: ContractToCreatePayAsYouGoTimeBasedJobTitle
    :param seniority: Job seniority level. Please use the seniority levels endpoint to retrieve the list of seniority levels., defaults to None
    :type seniority: ContractToCreatePayAsYouGoTimeBasedSeniority, optional
    :param notice_period: Days before to notice the termination of contract for eather party., defaults to None
    :type notice_period: float, optional
    :param who_reports: Flag to indicate who is supposed to provide regular reports, defaults to None
    :type who_reports: ContractWhoReportsEnum, optional
    :param meta: Additional custom info about a contract
    :type meta: MetaDataOfContractToCreate
    :param external_id: External Id., defaults to None
    :type external_id: str, optional
    :param worker: Worker properties, defaults to None
    :type worker: ContractToCreatePayAsYouGoTimeBasedWorker, optional
    :param type_: type_
    :type type_: ContractToCreatePayAsYouGoTimeBasedType
    :param start_date: Short date in format ISO-8601 (YYYY-MM-DD). For example: 2022-12-31.
    :type start_date: str
    :param compensation_details: compensation_details
    :type compensation_details: ContractToCreatePayAsYouGoTimeBasedCompensationDetails
    """

    def __init__(
        self,
        title: str,
        client: ContractToCreatePayAsYouGoTimeBasedClient,
        job_title: ContractToCreatePayAsYouGoTimeBasedJobTitle,
        meta: MetaDataOfContractToCreate,
        type_: ContractToCreatePayAsYouGoTimeBasedType,
        start_date: str,
        compensation_details: ContractToCreatePayAsYouGoTimeBasedCompensationDetails,
        country_code: str = None,
        state_code: str = None,
        scope_of_work: str = None,
        special_clause: str = None,
        termination_date: str = None,
        seniority: ContractToCreatePayAsYouGoTimeBasedSeniority = None,
        notice_period: float = None,
        who_reports: ContractWhoReportsEnum = None,
        external_id: str = None,
        worker: ContractToCreatePayAsYouGoTimeBasedWorker = None,
    ):
        self.title = title
        if country_code is not None:
            self.country_code = self._pattern_matching(
                country_code, "^[A-Z]{2}$", "country_code"
            )
        if state_code is not None:
            self.state_code = state_code
        if scope_of_work is not None:
            self.scope_of_work = scope_of_work
        if special_clause is not None:
            self.special_clause = special_clause
        if termination_date is not None:
            self.termination_date = self._pattern_matching(
                termination_date, "^\d{4}-\d{2}-\d{2}$", "termination_date"
            )
        self.client = self._define_object(
            client, ContractToCreatePayAsYouGoTimeBasedClient
        )
        self.job_title = self._define_object(
            job_title, ContractToCreatePayAsYouGoTimeBasedJobTitle
        )
        if seniority is not None:
            self.seniority = self._define_object(
                seniority, ContractToCreatePayAsYouGoTimeBasedSeniority
            )
        if notice_period is not None:
            self.notice_period = notice_period
        if who_reports is not None:
            self.who_reports = self._enum_matching(
                who_reports, ContractWhoReportsEnum.list(), "who_reports"
            )
        self.meta = self._define_object(meta, MetaDataOfContractToCreate)
        if external_id is not None:
            self.external_id = external_id
        if worker is not None:
            self.worker = self._define_object(
                worker, ContractToCreatePayAsYouGoTimeBasedWorker
            )
        self.type_ = self._enum_matching(
            type_, ContractToCreatePayAsYouGoTimeBasedType.list(), "type_"
        )
        self.start_date = start_date
        self.compensation_details = self._define_object(
            compensation_details, ContractToCreatePayAsYouGoTimeBasedCompensationDetails
        )



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id"})
class Team(BaseModel):
    """Team

    :param id_: Unique identifier of this resource., defaults to None
    :type id_: str, optional
    :param name: The name of the team., defaults to None
    :type name: str, optional
    """

    def __init__(self, id_: str = None, name: str = None):
        if id_ is not None:
            self.id_ = id_
        if name is not None:
            self.name = name



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .alternate_email_item import AlternateEmailItem
from .worker_legal_entity import WorkerLegalEntity


@JsonMap({"id_": "id"})
class WorkerOfContract(BaseModel):
    """WorkerOfContract

    :param id_: Unique identifier of this resource., defaults to None
    :type id_: str, optional
    :param full_name: Worker's full name., defaults to None
    :type full_name: str, optional
    :param email: User's email address., defaults to None
    :type email: str, optional
    :param alternate_email: List of alternate email addresses., defaults to None
    :type alternate_email: List[AlternateEmailItem], optional
    :param nationality: Worker's nationality., defaults to None
    :type nationality: str, optional
    :param image: image, defaults to None
    :type image: str, optional
    :param legal_entity: legal_entity, defaults to None
    :type legal_entity: WorkerLegalEntity, optional
    :param date_of_birth: Long date-time format following ISO-8601, defaults to None
    :type date_of_birth: str, optional
    :param expected_email: Worker's email address., defaults to None
    :type expected_email: str, optional
    :param first_name: Worker's first name, defaults to None
    :type first_name: str, optional
    :param last_name: Worker's last name, defaults to None
    :type last_name: str, optional
    :param country: Worker's country (location not nationality), defaults to None
    :type country: str, optional
    """

    def __init__(
        self,
        id_: str = None,
        full_name: str = None,
        email: str = None,
        alternate_email: List[AlternateEmailItem] = None,
        nationality: str = None,
        image: str = None,
        legal_entity: WorkerLegalEntity = None,
        date_of_birth: str = None,
        expected_email: str = None,
        first_name: str = None,
        last_name: str = None,
        country: str = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if full_name is not None:
            self.full_name = full_name
        if email is not None:
            self.email = email
        if alternate_email is not None:
            self.alternate_email = self._define_list(
                alternate_email, AlternateEmailItem
            )
        if nationality is not None:
            self.nationality = nationality
        if image is not None:
            self.image = image
        if legal_entity is not None:
            self.legal_entity = self._define_object(legal_entity, WorkerLegalEntity)
        if date_of_birth is not None:
            self.date_of_birth = date_of_birth
        if expected_email is not None:
            self.expected_email = expected_email
        if first_name is not None:
            self.first_name = first_name
        if last_name is not None:
            self.last_name = last_name
        if country is not None:
            self.country = country



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .global_payroll_g2_n_report import GlobalPayrollG2NReport


@JsonMap({})
class GlobalPayrollG2NReportContainer(BaseModel):
    """GlobalPayrollG2NReportContainer

    :param data: data
    :type data: List[GlobalPayrollG2NReport]
    """

    def __init__(self, data: List[GlobalPayrollG2NReport]):
        self.data = self._define_list(data, GlobalPayrollG2NReport)
